# _domain.py
#
# openipmi controller handling for domains
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2005 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import gui_popup
import gui_setdialog
import OpenIPMI
import _entity
import _mc
import _saveprefs
import _sel
import _fru
import _conn
import _oi_logging

class InvalidDomainError(Exception):
    def __init__(self, value):
        self.value = value
    def __repr__(self):
        return repr(self.value)
    def __str__(self):
        return str(self.value)

class DomainOpHandler:
    def __init__(self, d, func, handler):
        self.d = d
        self.func = func
        self.handler = handler
        return

    def DoOp(self):
        self.rv = 0
        rv = self.d.domain_id.to_domain(self)
        if ((rv == 0) and (self.rv != None)):
            rv = self.rv
            pass
        return

    def domain_cb(self, domain):
        if (self.handler):
            self.rv = getattr(domain, self.func)(self.handler)
        else:
            self.rv = getattr(domain, self.func)()
        return


class DomainRefreshData:
    def __init__(self, d, func):
        self.d = d;
        self.item = None
        self.func = func
        return

    def SetItem(self, item):
        self.item = item
        return

    def DoUpdate(self):
        if (not self.item):
            return
        if (not self.d.domain_id):
            return
        self.d.domain_id.to_domain(self)
        return

    def domain_cb(self, domain):
        val = getattr(domain, self.func)()
        self.d.ui.set_item_text(self.item, str(val))
        return

    pass

class DomainFRUDevidGet:
    def __init__(self, d):
        self.d = d;
        gui_setdialog.SetDialog("FRU Information Device ID for" + str(d),
                                [ True, str(0x20), str(0), str(0), str(0),
                                  str(0) ],
                                6, self,
                                [ "Is Logical", "IPMB Address", "Device ID",
                                  "LUN", "Private Bus", "Channel" ])
        return

    def do_on_close(self):
        del(self.d)
        del(self.fru)
        return

    def ok(self, vals):
        self.is_logical = vals[0]
        self.ipmb = int(vals[1])
        self.devid = int(vals[2])
        self.lun = int(vals[3])
        self.private_bus = int(vals[4])
        self.channel = int(vals[5])
        rv = self.d.domain_id.to_domain(self)
        if (rv == 0):
            rv = self.err
            pass
        if (rv != 0):
            return ("Error in FRU device information: "
                    + OpenIPMI.get_error_string(rv))
        return

    def domain_cb(self, domain):
        self.fru = domain.fru_alloc(int(self.is_logical), self.ipmb,
                                    self.devid, self.lun, self.private_bus,
                                    self.channel, self)
        if (self.fru == None):
            self.err = OpenIPMI.einval
            pass
        else:
            self.err = 0
            pass
        return

    def fru_fetched(self, domain, fru, err):
        s = str(self.d) + "(" + str(self.is_logical) + ", "
        s += str(self.ipmb) + ", "
        s += str(self.devid) + ", "
        s += str(self.lun) + ", "
        s += str(self.private_bus) + ", "
        s += str(self.channel) + ")"
        if (err):
            self.d.ui.new_log("Error fetching FRU iformation for " + s +
                              ": " + OpenIPMI.get_error_string(err))
            return
        _fru.FruInfoDisplay(fru, s)
        return

    pass

class DomainSelSet(DomainRefreshData):
    def __init__(self, d):
        self.d = d;
        self.refr = DomainRefreshData(d, "get_sel_rescan_time")
        return

    def DoUpdate(self):
        self.refr.DoUpdate()
        return

    def SetItem(self, item):
        self.refr.SetItem(item)
        return

    def HandleMenu(self, event):
        gui_popup.popup(self.d.ui, event, [ [ "Modify Value", self.modval ] ])
        return

    def modval(self, event):
        self.init = True
        self.d.domain_id.to_domain(self)
        gui_setdialog.SetDialog("Set SEL Rescan Time for " + str(self.d),
                                [ str(self.sel_rescan_time) ],
                                1,
                                self)
        return

    def ok(self, val):
        self.ival = int(val[0])
        self.init = False
        rv = self.d.domain_id.to_domain(self)
        if (rv != 0):
            return ("Error setting SEL rescan time: "
                    + OpenIPMI.get_error_string(rv))
        return

    def domain_cb(self, domain):
        if self.init:
            self.sel_rescan_time = domain.get_sel_rescan_time()
        else:
            domain.set_sel_rescan_time(self.ival)
            self.d.sel_rescan_time = self.ival
            self.refr.DoUpdate()
            pass
        return

    pass
        
class DomainIPMBSet(DomainRefreshData):
    def __init__(self, d):
        self.d = d;
        self.refr = DomainRefreshData(d, "get_ipmb_rescan_time")
        return

    def DoUpdate(self):
        self.refr.DoUpdate()
        return

    def SetItem(self, item):
        self.refr.SetItem(item)
        return

    def HandleMenu(self, event):
        gui_popup.popup(self.d.ui, event, [ [ "Modify Value", self.modval ] ])
        return

    def modval(self, event):
        self.init = True
        self.d.domain_id.to_domain(self)
        gui_setdialog.SetDialog("Set IPMB Rescan Time for " + str(self.d),
                                [ str(self.ipmb_rescan_time) ],
                                1,
                                self)
        return

    def ok(self, val):
        self.ival = int(val[0])
        self.init = False
        rv = self.d.domain_id.to_domain(self)
        if (rv == 0):
            rv = self.err
            pass
        if (rv != 0):
            return ("Error setting IPMB rescan time: "
                    + OpenIPMI.get_error_string(rv))
        return

    def domain_cb(self, domain):
        if self.init:
            self.ipmb_rescan_time = domain.get_ipmb_rescan_time()
        else:
            self.err = domain.set_ipmb_rescan_time(self.ival)
            self.d.ipmb_rescan_time = self.ival
            self.refr.DoUpdate()
            pass
        return

    pass


class DomainSaver:
    def __init__(self, d, doc, elem):
        self.d = d
        self.doc = doc
        self.elem = elem
        elem.setAttribute("name", self.d.name)
        self.d.domain_id.to_domain(self)
        if (hasattr(self.d, "ipmb_rescan_time")):
            e = doc.createElement("IPMB_rescan_time")
            e.setAttribute("time", str(self.d.ipmb_rescan_time))
            elem.appendChild(e)
            pass
        if (hasattr(self.d, "sel_rescan_time")):
            e = doc.createElement("SEL_rescan_time")
            e.setAttribute("time", str(self.d.sel_rescan_time))
            elem.appendChild(e)
            pass
        return

    def domain_cb(self, domain):
        domain.iterate_connections(self)
        return

    def domain_iter_connection_cb(self, domain, connum):
        args = domain.get_connection_args(connum)
        if (args == None):
            return
        celem = self.doc.createElement("connection")
        celem.setAttribute("contype", args.get_type())
        i = 0
        rv = 0
        while (rv == 0):
            name = [ "" ]
            vtype = [ "" ]
            vrange = [ "" ]
            vhelp = [ "" ]
            value = [ "" ]
            rv = args.get_val(i, name, vtype, vhelp, value, vrange)
            if (rv == 0):
                if (value[0] != None):
                    celem.setAttribute(name[0], value[0])
                    pass
                pass
            i += 1
            pass
        self.elem.appendChild(celem)

class Domain:
    def __init__(self, mainhandler, domain):
        name = domain.get_name()
        if (mainhandler.domains.has_key(name)):
            domain.close()
            raise InvalidDomainError("Domain name already exists")
        self.name = name
        self.mainhandler = mainhandler
        self.ui = mainhandler.ui
        self.entities = { }
        self.mcs = { }
        self.connections = { }
        self.domain_id = domain.get_id()
        
        mainhandler.domains[name] = self
        
        self.ui.add_domain(self)

        self.refreshers = [ ]
        self.add_refr_item("SEL Count", DomainRefreshData(self, "sel_count"))
        self.add_refr_item("SEL Entries Used",
                           DomainRefreshData(self, "sel_entries_used"))
        self.add_refr_item("IPMB Rescan Time", DomainIPMBSet(self))
        self.add_refr_item("SEL Rescan Time", DomainSelSet(self))
        self.add_refr_item("GUID", DomainRefreshData(self, "get_guid"))
        self.add_refr_item("Type", DomainRefreshData(self, "get_type"))
        
        self.first_conn = False
        self.any_con_up = False
        self.ui.incr_item_critical(self.treeroot)

        domain.add_connect_change_handler(self)
        return

    def __str__(self):
        return self.name

    def add_refr_item(self, name, refr):
        item = self.ui.prepend_item(self, name, None, refr)
        refr.SetItem(item)
        self.refreshers.append(refr)
        return refr
        
    def getTag(self):
        return "domain"

    def SaveInfo(self, doc, elem):
        DomainSaver(self, doc, elem)
        return

    def HandleExpand(self, event):
        for i in self.refreshers:
            i.DoUpdate()
            pass
        return

    def HandleMenu(self, event):
        gui_popup.popup(self.ui, event,
                        [ [ "Close",        self.CloseMenuHandler ],
                          [ "Reread SELs",  self.RereadSelsHandler ],
                          [ "Display SELs", self.DisplaySelsHandler ],
                          [ "Rescan IPMB",  self.RescanIPMBHandler ],
                          [ "Fetch FRU Info", self.FetchFruInfoHandler ] ])
        return

    def FetchFruInfoHandler(self, event):
        DomainFRUDevidGet(self);
        return

    def CloseMenuHandler(self, event):
        self.domain_id.to_domain(self)
        return

    def RereadSelsHandler(self, event):
        dop = DomainOpHandler(self, "reread_sels", None)
        dop.DoOp()
        return

    def DisplaySelsHandler(self, event):
        _sel.DomainSELDisplay(self.domain_id)
        return

    def RescanIPMBHandler(self, event):
        dop = DomainOpHandler(self, "start_full_ipmb_scan", None)
        dop.DoOp()
        return

    def Connect(self):
        self.already_up = False
        self.domain_id = OpenIPMI.open_domain3(self.name, [], self.connection,
                                               self, self)
        del self.connection
        if (self.domain_id == None):
            raise InvalidDomainError("Open domain failed, invalid parms")
        return

    def conn_change_cb(self, domain, err, connum, portnum, anything_connected):
        if (err):
            self.ui.new_log("Connection error for " + self.name
                            + "(" + str(connum) + "," + str(portnum) + "): "
                            + OpenIPMI.get_error_string(err))
            pass
        if (not self.first_conn):
            self.first_conn = True
            self.ipmb_rescan_time = domain.get_ipmb_rescan_time()
            self.sel_rescan_time = domain.get_sel_rescan_time()
            domain.iterate_connections(self)
            pass
        if (connum not in self.connections):
            self.ui.new_log("Got connection for invalid connection number")
            return
        self.connections[connum].SetPortUp(domain, portnum, err)
        any_con_up = False
        for c in self.connections.itervalues():
            any_con_up = c.IsUp() or any_con_up
            pass
        if (any_con_up):
            if (not self.any_con_up):
                self.ui.decr_item_critical(self.treeroot)
                pass
            pass
        else:
            if (self.any_con_up):
                self.ui.incr_item_critical(self.treeroot)
                pass
            pass
        self.any_con_up = any_con_up
        return

    def domain_iter_connection_cb(self, domain, conn):
        _conn.Connection(domain, self, conn)
        return
    
    def connected(self, domain):
        domain.add_entity_update_handler(self)
        domain.add_mc_update_handler(self)
        return

    def find_or_create_entity(self, entity):
        ename = entity.get_name()
        if (ename in self.entities):
            return self.entities[ename];
        else:
            return _entity.Entity(self, entity)
        return
        
    def find_entity_byname(self, ename):
        if (ename in self.entities):
            return self.entities[ename];
        else:
            return None
        return
        
    def entity_update_cb(self, op, domain, entity):
        if (op == "added"):
            e = self.find_or_create_entity(entity)
        elif (op == "deleted"):
            self.entities[entity.get_name()].remove()
        else:
            e = self.find_or_create_entity(entity)
            e.Changed(entity)
            pass
        return
        
    def find_or_create_mc(self, mc):
        mname = mc.get_name()
        if (mname in self.mcs):
            return self.mcs[mname];
        else:
            return _mc.MC(self, mc)
        return
        
    def mc_update_cb(self, op, domain, mc):
        if (op == "added"):
            if (mc.get_name() not in self.mcs):
                _mc.MC(self, mc)
                pass
            pass
        elif (op == "deleted"):
            if (mc.get_name() in self.mcs):
                self.mcs[mc.get_name()].remove()
                pass
            pass
        else:
            m = self.find_or_create_mc(mc)
            m.Changed(mc)
            pass
        return
        
    def domain_cb(self, domain):
        domain.close(self)
        return

    def domain_close_done_cb(self):
        return
        
    def remove(self):
        if (self.domain_id != None):
            self.domain_id.to_domain(self)
            pass
        del self.mainhandler.domains[self.name]
        self.ui.remove_domain(self)
        for c in self.connections.itervalues():
            c.remove()
            pass
        self.ui = None
        return

defaultDomains = [ ]

# Catch new and removed domains here
class DomainWatcher:
    def __init__(self, mainhandler):
        self.mainhandler = mainhandler
        return

    def domain_change_cb(self, change, domain):
        if (change == "added"):
            Domain(self.mainhandler, domain)
            pass
        elif (change == "deleted"):
            name = domain.get_name();
            if (name in self.mainhandler.domains):
                self.mainhandler.domains[name].remove()
                pass
            pass
        return

class OtherDomainInfo:
    pass

class DomainInfoSetup:
    def __init__(self, other, domain_id):
        self.other = other
        domain_id.to_domain(self)
        return

    def domain_cb(self, domain):
        if (hasattr(self.other, "ipmb_rescan_time")):
            domain.set_ipmb_rescan_time(self.other.ipmb_rescan_time)
            pass
        if (hasattr(self.other, "sel_rescan_time")):
            domain.set_sel_rescan_time(self.other.sel_rescan_time)
            pass
        return

def RestoreDomains(mainhandler):
    for i in defaultDomains:
        name = i[0]
        attrhashes = i[1]
        other = i[2]
        connects = [ ]
        for attrhash in attrhashes:
            if ("contype" not in attrhash):
                continue
            args = OpenIPMI.alloc_empty_args(str(attrhash["contype"]))
            if (args == None):
                continue
            for attr in attrhash.items():
                vname = str(attr[0])
                if (vname == "contype"):
                    continue
                value = str(attr[1])
                args.set_val(0, vname, value)
                pass
            connects.append(args)
            pass
        domain_id = OpenIPMI.open_domain3(name, [], connects, None, None)
        if (domain_id != None):
            DomainInfoSetup(other, domain_id)
            pass
        else:
            _oi_logging.error("Error making domain connection for " + name)
            pass
        pass
    return


class _DomainRestore(_saveprefs.RestoreHandler):
    def __init__(self):
        _saveprefs.RestoreHandler.__init__(self, "domain")
        return

    def restore(self, node):
        name = str(node.getAttribute("name"));
        if (name == ""):
            return
        connects = [ ]
        other = OtherDomainInfo()
        for c in node.childNodes:
            if (c.nodeType == c.ELEMENT_NODE):
                if (c.nodeName == "connection"):
                    attrhash = { }
                    for i in range(0, c.attributes.length):
                        attr = c.attributes.item(i)
                        attrhash[attr.nodeName] = attr.nodeValue
                        pass
                    connects.append(attrhash)
                    pass
                elif (c.nodeName == "IPMB_rescan_time"):
                    try:
                        other.ipmb_rescan_time = int(c.getAttribute("time"))
                    except:
                        _oi_logging.error("Error restoring IPMB rescan time"
                                          + " in a domain")
                        pass
                    pass
                elif (c.nodeName == "SEL_rescan_time"):
                    try:
                        other.sel_rescan_time = int(c.getAttribute("time"))
                    except:
                        _oi_logging.error("Error restoring SEL rescan time"
                                          + " in a domain")
                        pass
                    pass
                pass
            pass
        defaultDomains.append([name, connects, other])
        return
    pass

_DomainRestore()



# _mc_user.py
#
# openipmi GUI handling for MC users
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2005 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import OpenIPMI
import gui_popup
import gui_treelist
import gui_setdialog
import _oi_logging
import _mc_lanparm

class BoolSetter:
    def __init__(self, mcusers, user, setter):
        self.mcusers = mcusers
        self.setter = setter
        self.user = user
        return
    
    def SetItem(self, item):
        self.item = item
        return
    
    def HandleMenu(self, event, eitem, point):
        gui_popup.popup(self.mcusers, event,
                        [ ("Toggle Value", self.togglevalue) ],
                        point)
        return

    def togglevalue(self, event):
        val = str(self.mcusers.GetColumn(self.item, 1))
        if (val == "True") or (val == "true"):
            val = "false"
            bval = 0
        else:
            val = "true"
            bval = 1
            pass
        rv = self.setter(bval)
        if (rv):
            self.mcusers.SetError("Could not toggle value: "
                                   + OpenIPMI.get_error_string(rv), 0)
            return
        self.user.changed = True
        self.mcusers.SetColumn(self.item, val, 1);
        return

    pass

class IntSetter:
    def __init__(self, mcusers, user, setter, name, currval):
        self.mcusers = mcusers
        self.setter = setter
        self.name = name
        self.currval = currval
        self.user = user
        return
    
    def SetItem(self, item):
        self.item = item
        return
    
    def HandleMenu(self, event, eitem, point):
        gui_popup.popup(self.mcusers, event,
                        [ ("Modify Value", self.modval) ],
                        point)
        return

    def modval(self, event):
        gui_setdialog.SetDialog("Set Value for " + self.name,
                                [ str(self.currval) ],
                                1,
                                self)
        return

    def ok(self, vals):
        val = int(vals[0])
        rv = self.setter(val)
        if (rv):
            return ("Could not set value: "
                    + OpenIPMI.get_error_string(rv))
        self.user.changed = True
        self.mcusers.SetColumn(self.item, str(val), 1)
        self.currval = val
        return

    pass

class StrSetter:
    def __init__(self, mcusers, user, setter, name, currval, prompt):
        self.mcusers = mcusers
        self.setter = setter
        self.name = name
        self.currval = currval
        self.user = user
        self.prompt = prompt
        return
    
    def SetItem(self, item):
        self.item = item
        return
    
    def HandleMenu(self, event, eitem, point):
        gui_popup.popup(self.mcusers, event,
                        [ (self.prompt, self.modval) ],
                        point)
        return

    def modval(self, event):
        gui_setdialog.SetDialog(self.prompt + " for " + self.name,
                                [ str(self.currval) ],
                                1,
                                self)
        return

    def ok(self, vals):
        val = str(vals[0])
        rv = self.setter(val)
        if (rv):
            return ("Could not set value: "
                    + OpenIPMI.get_error_string(rv))
        self.user.changed = True
        self.mcusers.SetColumn(self.item, str(val), 1)
        self.currval = val
        return

    pass

def GetPrivilegeString(val):
    if (val == 15):
        return "NO ACCESS"
    else:
        return OpenIPMI.privilege_string(val)
    return

class PrivSetter:
    def __init__(self, mcusers, user, setter):
        self.mcusers = mcusers
        self.setter = setter
        self.user = user
        return
    
    def SetItem(self, item):
        self.item = item
        return
    
    def HandleMenu(self, event, eitem, point):
        gui_popup.popup(self.mcusers, event,
                        [ ("Callback", self.callback),
                          ("User", self.handleuser),
                          ("Operator", self.operator),
                          ("Admin", self.admin),
                          ("OEM", self.oem),
                          ("NO ACCESS", self.noaccess) ],
                        point)
        return

    def setval(self, val):
        rv = self.setter(val)
        if (rv):
            mcusers.SetError("Could not set value: "
                             + OpenIPMI.get_error_string(rv))
            return
        self.user.changed = True
        self.mcusers.SetColumn(self.item, GetPrivilegeString(val), 1)
        return
        
    def callback(self, event):
        self.setval(OpenIPMI.PRIVILEGE_CALLBACK)
        return

    def handleuser(self, event):
        self.setval(OpenIPMI.PRIVILEGE_USER)
        return

    def operator(self, event):
        self.setval(OpenIPMI.PRIVILEGE_OPERATOR)
        return

    def admin(self, event):
        self.setval(OpenIPMI.PRIVILEGE_ADMIN)
        return

    def oem(self, event):
        self.setval(OpenIPMI.PRIVILEGE_OEM)
        return

    def noaccess(self, event):
        self.setval(15)
        return

    pass

def IntToBoolStr(v):
    if (v):
        return "true"
    else:
        return "false"
    return

class SetUserHandler:
    def __init__(self, mcusers, num):
        self.mcusers = mcusers
        self.num = num
        return

    def mc_channel_set_user_cb(self, mc, err):
        self.mcusers.user_set(mc, err, self.num)
        return

    pass

class MCUsers(gui_treelist.TreeList):
    def __init__(self, mc, channel, max_users, enabled_users, fixed_users,
                 users):
        gui_treelist.TreeList.__init__(self, "user info for " + mc.get_name()
                                       + " channel " + str(channel),
                                       "Users",
                                       [ ("Name", 300), ("Value", 400) ] )
        
        self.mc_id = mc.get_id()
        self.count = 0;
        self.users = users
        self.channel = channel
        self.in_save = False

        item = self.add_data(self.treeroot, "Max Users", [ str(max_users) ])
        item = self.add_data(self.treeroot, "Enabled Users",
                           [ str(enabled_users) ])
        item = self.add_data(self.treeroot, "Fixed Users", [str(fixed_users)])
        
        for u in users:
            v = [ 0 ]
            rv = u.get_num(v)
            u.changed = False
            num = v[0]
            if (rv == 0):
                u.num = num
                nm = u.get_name()
                setter = StrSetter(self, u, u.set_password_auto,
                                   "Password", "", "Set Password")
                if (nm):
                    nm = str(nm)
                    us = self.add_data(self.treeroot, str(num), [nm], setter)
                else:
                    us = self.add_data(self.treeroot, str(num), [], setter)
                    nm = ""
                    pass
                setter.SetItem(us)
                if (num > fixed_users):
                    setter = StrSetter(self, u, u.set_name, "Name", nm,
                                       "Modify Value")
                else:
                    setter = None;
                    pass
                item = self.add_data(us, "Name", [nm], setter)

                rv = u.get_enable(v)
                if (rv == 0):
                    s = IntToBoolStr(v[0])
                else:
                    s = "?"
                    pass
                item = self.add_data(us, "Enabled", [s],
                                     BoolSetter(self, u, u.set_enable))

                rv = u.get_link_auth_enabled(v)
                if (rv == 0):
                    s = IntToBoolStr(v[0])
                else:
                    s = "?"
                    pass
                item = self.add_data(us, "Link Auth Enabled", [s],
                                     BoolSetter(self, u,
                                                u.set_link_auth_enabled))

                rv = u.get_msg_auth_enabled(v)
                if (rv == 0):
                    s = IntToBoolStr(v[0])
                else:
                    s = "?"
                    pass
                item = self.add_data(us, "Msg Auth Enabled", [s],
                                     BoolSetter(self, u,
                                                u.set_msg_auth_enabled))

                rv = u.get_access_cb_only(v)
                if (rv == 0):
                    s = IntToBoolStr(v[0])
                else:
                    s = "?"
                    pass
                item = self.add_data(us, "Access Callback Only", [s],
                                     BoolSetter(self, u, u.set_access_cb_only))

                rv = u.get_privilege_limit(v)
                if (rv == 0):
                    s = GetPrivilegeString(v[0])
                else:
                    s = "?"
                    pass
                item = self.add_data(us, "Privilege Limit", [s],
                                     PrivSetter(self, u,
                                                u.set_privilege_limit))

                rv = u.get_session_limit(v)
                if (rv == 0):
                    s = str(v[0])
                else:
                    s = "?"
                    v[0] = 0
                    pass
                item = self.add_data(us, "Session Limit", [s],
                                     IntSetter(self, u, u.set_session_limit,
                                               "Session Limit", v[0]))
                pass
            pass
        
        self.AfterDone()
        return

    def cancel(self):
        if (self.in_save):
            return
        self.Close()
        return
    
    def save(self):
        if (self.in_save):
            return
        self.mc_id.to_mc(self)
        return

    def mc_cb(self, mc):
        self.errnum = 0
        self.waitcount = 0
        self.SetError("")
        for u in self.users:
            if (u.changed):
                rv = mc.set_user(u, self.channel, u.num,
                                 SetUserHandler(self, u.num))
                if (rv):
                    self.SetError("Error setting user "
                                  + str(u.num)  + ": "
                                  + OpenIPMI.get_error_string(rv))
                    self.errnum += 1
                else:
                    self.waitcount += 1
                    pass
                pass
            pass
        if ((self.errnum == 0) and (self.waitcount == 0)):
            self.Close()
        elif (self.waitcount > 0):
            self.in_save = True
        return

    def user_set(self, mc, err, num):
        if (err):
            self.SetError("Error setting user " + str(num) + ": "
                          + OpenIPMI.get_error_string(err))
            self.errnum += 1
            pass
        self.waitcount -= 1
        if (self.waitcount <= 0):
            self.in_save = False;
            if (self.errnum == 0):
                self.Close()
                pass
            pass
        return
    
    def do_on_close(self):
        self.users = None
        return

    pass



# _mc_lanparm.py
#
# openipmi GUI handling for MC LAN parms
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2005 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import sys
import OpenIPMI
import _oi_logging
import gui_errstr
import gui_list
import gui_popup
import gui_setdialog

class MCLPData:
    def __init__(self, glist, lpc, parm, aidx, pname, ptype, origval):
        self.glist = glist
        self.lpc = lpc
        self.parm = parm
        self.aidx = aidx
        self.pname = pname
        self.ptype = ptype
        self.origval = origval
        self.currval = origval
        return

    def SetItem(self, idx):
        self.idx = idx;
        return

    def HandleMenu(self, event, idx, point):
        if (self.ptype == "bool"):
            menul = [ ("Toggle Value", self.togglevalue) ]
        elif (self.ptype == "enum"):
            menul = [ ]
            nval = [ 0 ]
            sval = [ "" ]
            val = 0;
            while (val != -1):
                rv = OpenIPMI.lanconfig_enum_val(self.parm, val, nval, sval)
                if (rv == 0):
                    menul.append( (sval[0] + " (" + str(val) + ")",
                                   self.setenum,
                                   val) )
                    pass
                val = nval[0];
                pass
            pass
        else:
            menul = [ ("Set Value", self.setvalue) ]
            pass
        gui_popup.popup(self.glist, event, menul, point)
        return

    def ok(self, vals):
        rv = self.lpc.set_val(self.parm, self.aidx, self.ptype, str(vals[0]))
        if (rv != 0):
            self.glist.SetError("Invalid data value: "
                                + OpenIPMI.get_error_string(rv))
            return
        self.currval = vals[0]
        self.glist.SetColumn(self.idx, 1, vals[0])
        return
    
    def setvalue(self, event):
        gui_setdialog.SetDialog("Set value for " + self.pname,
                                [ self.currval ], 1, self)
        return

    def setenum(self, val):
        rv = self.lpc.set_val(self.parm, self.aidx, "integer", str(val))
        if (rv != 0):
            self.glist.SetError("Could not set value to " + str(val) + ": "
                                + OpenIPMI.get_error_string(rv))
            return
        self.currval = val
        nval = [ 0 ]
        sval = [ "" ]
        OpenIPMI.lanconfig_enum_val(self.parm, val, nval, sval)
        self.glist.SetColumn(self.idx, 1, sval[0])
        return
    
    def togglevalue(self, event):
        if (self.currval == "true"):
            newval = "false"
        else:
            newval = "true"
            pass
        rv = self.lpc.set_val(self.parm, self.aidx, self.ptype, newval)
        if (rv != 0):
            self.glist.SetError("Could not toggle value: "
                                + OpenIPMI.get_error_string(rv))
            return
            
        self.currval = newval
        self.glist.SetColumn(self.idx, 1, newval)
        return

    pass

class MCLanParm(gui_list.List):
    def __init__(self, m, lp, lpc, channel):
        gui_list.List.__init__(self,
                               ("LANPARMS for " + m.name + " channel "
                                + str(channel)),
                               [ ("Name", 250), ("Value", 250) ])
        self.lp = lp
        self.lpc = lpc
        self.channel = channel
        
        i = 0
        j = 0
        rv = True
        v = [ 0 ]
        while (rv):
            lastv = v[0]
            rv = lpc.get_val(i, v)
            if (rv):
                vals = rv.split(" ", 2)
                if (len(vals) == 3):
                    # Valid parm
                    if (vals[1] == "integer"):
                        w = [ 0 ]
                        x = [ "" ]
                        err = OpenIPMI.lanconfig_enum_val(i, 0, w, x)
                        if (err != OpenIPMI.enosys):
                            vals[1] = "enum"
                            pass
                        pass
                    
                    data = MCLPData(self, lpc, i, lastv,
                                    vals[0], vals[1], vals[2])

                    if (v[0] == 0):
                        title = vals[0]
                    else:
                        x = [ "" ]
                        err = OpenIPMI.lanconfig_enum_idx(i, lastv, x)
                        if (err):
                            title = vals[0] + "[" + str(lastv) + "]"
                        else:
                            title = vals[0] + "[" + x[0] + "]"
                            pass
                        pass
                    if (vals[1] == "enum"):
                        nval = [ 0 ]
                        sval = [ "" ]
                        OpenIPMI.lanconfig_enum_val(i, int(vals[2]),
                                                    nval, sval)
                        value = sval[0]
                        pass
                    else:
                        value = vals[2]
                        pass
                    
                    self.add_data(title, [ value ], data)

                    j += 1
                    if (v[0] == 0):
                        i += 1
                        pass
                    if (v[0] == -1):
                        i += 1
                        v[0] = 0
                        pass
                    pass
                else:
                    v[0] = 0
                    i += 1
                    pass
                pass
            pass

        self.AfterDone()
        return

    def save(self):
        rv = self.lp.set_config(self.lpc)
        if (rv != 0):
            self.SetError("Error setting config: "
                          + OpenIPMI.get_error_string(rv))
            return

        # Don't forget to set self.lp to None when done so OnClose
        # doesn't clear it again
        self.lp = None
        self.Close()
        return

    def cancel(self):
        self.Close()
        return

    def do_on_close(self):
        # Do it here, not in cancel, to handle closing the window without
        # clicking on "save" or "cancel"
        if (self.lp):
            self.lp.clear_lock(self.lpc)
            self.lp = None
            pass
        self.lpc = None
        return
    
    pass



# gui.py
#
# main openipmi GUI handling
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2005 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
import Tix
import OpenIPMI
import _saveprefs
import _oi_logging
import gui_domainDialog
import gui_errstr
import gui_cmdwin
import gui_list
import gui_popup

init_treenamewidth = 150
init_sashposition = 400
init_isashposition = 300
init_bsashposition = 350
init_windowwidth = 800
init_windowheight = 700
init_logevents = False
init_fullevents = False
init_impt_objs = [ ]

try:
    winsys = self.tk.eval("return [ tk windowingsystem ]")
    pass
except:
    # Assume x11
    winsys = "x11"
    pass

refresh_timer_time = 10000

class IPMITreeDummyItem:
    def __init__(self, treestr):
        self.treestr = treestr
        return

    pass

class IPMICloser:
    def __init__(self, ui, count):
        self.ui = ui
        self.count = count
        return

    def domain_cb(self, domain):
        domain.close(self)
        return

    def domain_close_done_cb(self):
        self.count = self.count - 1
        return

    def wait_done(self):
        while (self.count > 0):
            OpenIPMI.wait_io(1000)
            pass
        gui_cmdwin.init_history = self.ui.cmdwindow.history
        return
    
    pass

class ImptObj:
    def __init__(self, gui, type, name, obj):
        self.gui = gui
        self.type = type
        self.name = name
        self.obj = obj
        return

    def HandleMenu(self, event, key, point):
        data = self.obj
        if (data != None) and (hasattr(data, "HandleMenu")):
            data.HandleMenu(event)
            pass
        else:
            menul = [ [ "Remove from watch values", self.remove_impt ] ]
            gui_popup.popup(self.gui, event, menul)
        return
        
    def remove_impt(self, event):
        self.gui.remove_impt_data(self)
        return

    pass

class IPMIGUI(Tix.Frame):
    def __init__(self, top, mainhandler):
        Tix.Frame.__init__(self, top, bd=2, relief=Tix.RAISED)

        self.pack(side=Tix.TOP, fill=Tix.BOTH, expand=1)

        self.top = top

        self.mainhandler = mainhandler

        self.inactive_style = Tix.DisplayStyle(Tix.TEXT, fg="dark grey",
                                               selectforeground="dark grey",
                                               selectbackground="beige",
                                               refwindow=top)
        self.active_style = Tix.DisplayStyle(Tix.TEXT, fg="black",
                                             selectforeground="black",
                                             selectbackground="beige",
                                             refwindow=top)
        self.critical_style = Tix.DisplayStyle(Tix.TEXT, fg="blue",
                                               selectforeground="blue",
                                               selectbackground="beige",
                                               refwindow=top)
        self.severe_style = Tix.DisplayStyle(Tix.TEXT, fg="red",
                                             selectforeground="red",
                                             selectbackground="beige",
                                             refwindow=top)
        self.warn_style = Tix.DisplayStyle(Tix.TEXT, fg="burlywood4",
                                           selectforeground="burlywood4",
                                           selectbackground="beige",
                                           refwindow=top)
        
        self.logeventsv = Tix.IntVar()
        self.logeventsv.set(init_logevents)
        self.logevents = init_logevents
        self.fulleventsv = Tix.IntVar()
        self.fulleventsv.set(init_fullevents)
        OpenIPMI.cmdlang_set_evinfo(self.fulleventsv.get())
        
        fileb = Tix.Menubutton(self, text="File", underline=0, takefocus=0)
        filemenu = Tix.Menu(fileb, tearoff=0)
        fileb["menu"] = filemenu
        filemenu.add_command(label="Exit", underline=1, accelerator="Ctrl+Q",
                             command = lambda self=self: self.quit() )
        top.bind_all("<Control-Q>", self.quit)
        top.bind_all("<Control-q>", self.quit)
        filemenu.add_command(label="Open Domain", underline=1,
                             accelerator="Ctrl+O",
                             command = lambda self=self: self.openDomain() )
        top.bind_all("<Control-O>", self.openDomain)
        top.bind_all("<Control-o>", self.openDomain)
        filemenu.add_command(label="Save Prefs", underline=1,
                             accelerator="Ctrl+S",
                             command = lambda self=self: self.savePrefs() )
        top.bind_all("<Control-S>", self.savePrefs)
        top.bind_all("<Control-s>", self.savePrefs)

        viewb = Tix.Menubutton(self, text="View", underline=0, takefocus=0)
        viewmenu = Tix.Menu(viewb, tearoff=0)
        viewb["menu"] = viewmenu
        viewmenu.add_command(label="Expand All", underline=1,
                             accelerator="Ctrl+E",
                             command = lambda self=self: self.ExpandAll() )
        top.bind_all("<Control-E>", self.ExpandAll)
        top.bind_all("<Control-e>", self.ExpandAll)
        viewmenu.add_command(label="Collapse All", underline=1,
                             accelerator="Ctrl+C",
                             command = lambda self=self: self.CollapseAll() )
        top.bind_all("<Control-C>", self.CollapseAll)
        top.bind_all("<Control-c>", self.CollapseAll)

        setb = Tix.Menubutton(self, text="Settings", underline=0, takefocus=0)
        viewmenu = Tix.Menu(setb, tearoff=0)
        setb["menu"] = viewmenu
        viewmenu.add_checkbutton(label="Enable Events", underline=0,
                                 command=lambda w=self: w.EnableEvents(),
                                 variable=self.logeventsv)
        viewmenu.add_checkbutton(label="Full Event Info", underline=0,
                                 command=lambda w=self: w.FullEventInfo(),
                                 variable=self.fulleventsv)

        vpane = Tix.PanedWindow(self, orientation="vertical",
                                width=init_windowwidth,
                                height=init_windowheight)
        self.vpane = vpane
        objevpane = vpane.add("objectsevents", size=init_sashposition)
        imptobjpane = vpane.add("importantobjects",
                                size = init_isashposition - init_sashposition)
        cmdpane = vpane.add("command")
        hpane = Tix.PanedWindow(objevpane, orientation="horizontal")
        self.hpane = hpane
        objpane = hpane.add("objects", size=init_bsashposition)
        evpane = hpane.add("events")

        self.tree = Tix.Tree(objpane, options="hlist.columns 2")
        # FIXME: This doesn't work, and I don't know why
        self.tree.hlist.configure(selectbackground="beige")
        self.tree.hlist.add("D", itemtype=Tix.TEXT, text="Domains")
        self.tree.setmode("D", "none")
        self.treedata = { }
        self.treedata["D"] = IPMITreeDummyItem("D")
        self.setup_item("D", active=True)
        self.tree.pack(side=Tix.TOP, fill=Tix.BOTH, expand=1)
        self.tree.hlist.bind("<Button-3>", self.TreeMenu)
        
        self.tree.hlist.bind("<MouseWheel>", self.Wheel)
        if (winsys == "x11"):
            self.tree.hlist.bind("<Button-4>", self.ButtonUp)
            self.tree.hlist.bind("<Button-5>", self.ButtonDown)
            pass

        self.numloglines = 1
        self.maxloglines = 1000
        self.logwindow = Tix.ScrolledText(evpane)
        self.logwindow.pack(side=Tix.TOP, fill=Tix.BOTH, expand=1)
        self.logwindow.text.insert("end", "GUI Log Window")

        self.imptobjs = gui_list.SubList(imptobjpane,
                                         ( ("Type", 50),
                                           ("Name", 200),
                                           ("Data", 200) ),
                                         options=("hlist.header 1"
                                                  + " hlist.itemtype text"
                                                  + " hlist.columns 3"
                                                  + " hlist.selectForeground black"
                                                  + " hlist.selectBackground beige"),
                                         width=0, height=0)
        self.imptobjs.pack(fill=Tix.BOTH, expand=1)
        
        self.errstr = gui_errstr.ErrStr(cmdpane)
        self.errstr.pack(side=Tix.TOP, fill=Tix.X, expand=1)

        self.cmdwindow = gui_cmdwin.CommandWindow(cmdpane, self)
        self.cmdwindow.pack(side=Tix.TOP, fill=Tix.BOTH, expand=1)

        hpane.pack(side=Tix.TOP, fill=Tix.BOTH, expand=1)

        vpane.pack(side=Tix.BOTTOM, fill=Tix.BOTH, expand=1)
        fileb.pack(side=Tix.LEFT)
        viewb.pack(side=Tix.LEFT)
        setb.pack(side=Tix.LEFT)
        
        self.itemval = 0

        self.in_destroy = False

        self.bind("<Destroy>", self.OnDestroy)

        self.impt_objs = { }
        self.impt_objs["control"] = { }
        self.impt_objs["sensor"] = { }
        self.impt_objs["entity"] = { }
        
        self.last_scan = None
        self.timer_timeout_ms = 200
        top.after(self.timer_timeout_ms, self.Timeout)

        for i in init_impt_objs:
            self.add_impt_data(i[0], i[1])
            pass
        return

    def Wheel(self, event):
        self.tree.hlist.yview("scroll", -(event.delta / 20), "units")
        return
    
    def ButtonUp(self, event):
        event.delta = 120
        self.Wheel(event);
        return
    
    def ButtonDown(self, event):
        event.delta = -120
        self.Wheel(event);
        return
    
    def ReportError(self, str):
        if (self.in_destroy):
            return
        self.errstr.SetError(str)
        return

    def find_impt_data(self, type, name):
        s = self.impt_objs[type]
        if name in s:
            return s[name]
        return None

    def add_impt_data(self, type, name, obj=None):
        if name in self.impt_objs[type]:
            return
        i = ImptObj(self, type, name, obj)
        self.impt_objs[type][name] = i
        i.key = self.imptobjs.Append(type, (name, ""), data=i)
        if (obj != None):
            obj.impt_data = i;
            self.setup_impt_data(obj.impt_data, obj)
            pass
        else:
            self.imptobjs.SetColumnStyle(i.key, 0, self.inactive_style)
        return

    def setup_impt_data(self, data, obj):
        data.obj = obj
        self.set_impt_active_change(obj)
        self.imptobjs.SetColumnStyle(obj.impt_data.key, 0, self.active_style)
        self.set_impt_data_text(obj)
        return
    
    def cleanup_impt_data(self, obj):
        self.imptobjs.SetColumnStyle(obj.impt_data.key, 1,
                                     self.inactive_style)
        self.imptobjs.SetColumnStyle(obj.impt_data.key, 0, self.inactive_style)
        self.imptobjs.SetColumn(obj.impt_data.key, 2, "")
        obj.impt_data.obj = None
        return
    
    def set_impt_style(self, obj, style):
        self.imptobjs.SetColumnStyle(obj.impt_data.key, 1, style)
        return
    
    def set_impt_active_change(self, obj):
        if (obj.active):
            self.imptobjs.SetColumnStyle(obj.impt_data.key, 1,
                                         self.active_style)
            pass
        else:
            self.imptobjs.SetColumnStyle(obj.impt_data.key, 1,
                                         self.inactive_style)
            pass
        return
        
    def set_impt_data_text(self, obj):
        if (obj.itemvalue != None):
            self.imptobjs.SetColumn(obj.impt_data.key, 2, obj.itemvalue)
            pass
        else:
            self.imptobjs.SetColumn(obj.impt_data.key, 2, "")
            pass
        return
    
    def remove_impt_data(self, data):
        obj = data.obj
        self.imptobjs.DelItem(data.key);
        del self.impt_objs[data.type][data.name]
        if (obj != None):
            obj.impt_data = None
            pass
        return

    def Timeout(self):
        if (self.in_destroy):
            return
        callcount = 0
        checkcount = 0
        if (self.last_scan != None):
            next = self.last_scan
        else:
            # Scan important objects first
            next = self.tree.hlist.info_next("D")
            for i in self.impt_objs.values():
                for j in i.values():
                    if (j.obj != None) and hasattr(j.obj, "DoUpdate"):
                        callcount = callcount + 1
                        j.obj.DoUpdate()
                        pass
                    checkcount = checkcount + 1
                    pass
                pass
            pass
        while (callcount < 100) and (checkcount < 1000) and (next != ""):
            if (self.tree.hlist.info_hidden(next) == "1"):
                # Not on the screen, ignore it
                next = self.tree.hlist.info_next(next)
                continue
            data = self.treedata[next]
            if (data != None) and (hasattr(data, "DoUpdate")):
                callcount = callcount + 1
                data.DoUpdate()
                pass
            next = self.tree.hlist.info_next(next)
            checkcount = checkcount + 1
            pass
            
        if (next != ""):
            self.last_scan = next
            self.top.after(self.timer_timeout_ms, self.Timeout)
        else:
            self.last_scan = None
            self.top.after(refresh_timer_time, self.Timeout)
            pass
        
        return
        
    def quit(self, event=None):
        self.mainhandler.destroy()
        return

    def OnDestroy(self, event):
        self.in_destroy = True
        self.closecount = len(self.mainhandler.domains)
        closer = IPMICloser(self, self.closecount)
        ds = self.mainhandler.domains.values()
        for v in ds:
            v.domain_id.to_domain(closer)
            pass
        closer.wait_done()
        return

    def openDomain(self, event=None):
        dialog = gui_domainDialog.OpenDomainDialog(self.mainhandler)
        return

    def savePrefs(self, event=None):
        self.mainhandler.savePrefs()
        return

    def ExpandItem(self, item):
        children = self.tree.hlist.info_children(item)
        for child in children:
            self.tree.open(child)
            self.ExpandItem(child)
            pass
        return
        
    def ExpandAll(self, event=None):
        self.tree.open("D")
        self.ExpandItem("D")
        return
        
    def CollapseItem(self, item):
        children = self.tree.hlist.info_children(item)
        for child in children:
            self.tree.close(child)
            self.ExpandItem(child)
            pass
        return
        
    def CollapseAll(self, event=None):
        self.CollapseItem("D")
        return
        
    def EnableEvents(self, event=None):
        self.logevents = self.logeventsv.get() != 0
        print "logevents = " + str(self.logevents)
        return
    
    def FullEventInfo(self, event=None):
        print "fullevents = " + str(self.fulleventsv.get())
        OpenIPMI.cmdlang_set_evinfo(self.fulleventsv.get())
        return
    
    def new_log(self, log):
        if (self.in_destroy):
            return
        # If we are at the bottom, then scroll the window, otherwise
        # don't do any scrolling
        (top, bottom) = self.logwindow.text.yview()
        doscroll = bottom == 1.0
        self.numloglines += log.count("\n") + 1
        self.logwindow.text.insert("end", "\n" + log)
        overrun = self.numloglines - self.maxloglines
        if (overrun > 0):
            self.logwindow.text.delete("1.0", str(overrun+1)+".0")
            self.numloglines -= overrun
            pass
        if (doscroll):
            self.logwindow.text.see("end")
        return

    def setup_item(self, item, active=False, type = None):
        data = self.treedata[item]
        data.active = active
        data.num_warning = 0
        data.num_severe = 0
        data.num_critical = 0
        data.itemvalue = None
        if (type != None):
            data.impt_data = self.find_impt_data(type, data.name_str)
            if (data.impt_data != None):
                self.setup_impt_data(data.impt_data, data)
                pass
            pass
        else:
            data.impt_data = None
            pass
        self.tree.hlist.item_create(item, 1, itemtype=Tix.TEXT, text="",
                                    style=self.active_style)
        if (not active):
            self.tree.hlist.item_configure(item, 0, style=self.inactive_style)
            pass
        else:
            self.tree.hlist.item_configure(item, 0, style=self.active_style)
            pass
        return

    def cleanup_item(self, item):
        if (self.in_destroy):
            return
        data = self.treedata[item]
        if (data == None):
            return
        if (data.impt_data != None):
            self.cleanup_impt_data(data)
            pass
        parent = self.parent_item(item)
        if (parent == None):
            return
        while (data.num_warning > 0):
            data.num_warning = data.num_warning - 1;
            self.decr_item_warning(parent); 
            pass
        while (data.num_severe > 0):
            data.num_severe = data.num_severe - 1;
            self.decr_item_severe(parent); 
            pass
        while (data.num_critical > 0):
            data.num_critical = data.num_critical - 1;
            self.decr_item_critical(parent); 
            pass
        return

    def add_domain(self, d):
        if (self.in_destroy):
            return
        d.name_str = str(d)
        item = "D." + str(self.itemval)
        self.itemval += 1
        d.treeroot = item
        self.tree.hlist.add(d.treeroot, itemtype=Tix.TEXT, text=d.name_str)
        self.tree.setmode(d.treeroot, "open")
        self.tree.close(d.treeroot)
        self.treedata[d.treeroot] = d
        self.setup_item(d.treeroot, active=True)
        
        lstr = d.treeroot + ".E"
        self.tree.hlist.add(lstr, itemtype=Tix.TEXT, text="Entities")
        self.tree.setmode(lstr, "none")
        self.tree.close(lstr)
        self.tree.hlist.hide_entry(lstr)
        self.treedata[lstr] = IPMITreeDummyItem(lstr)
        self.setup_item(lstr, active=True)
        
        lstr = d.treeroot + ".M"
        self.tree.hlist.add(lstr, itemtype=Tix.TEXT, text="MCs")
        self.tree.setmode(lstr, "none")
        self.tree.close(lstr)
        self.tree.hlist.hide_entry(lstr)
        self.treedata[lstr] = IPMITreeDummyItem(lstr)
        self.setup_item(lstr, active=True)
        
        lstr = d.treeroot + ".C"
        self.tree.hlist.add(lstr, itemtype=Tix.TEXT, text="Connections")
        self.tree.setmode(lstr, "none")
        self.tree.close(lstr)
        self.tree.hlist.hide_entry(lstr)
        self.treedata[lstr] = IPMITreeDummyItem(lstr)
        self.setup_item(lstr, active=True)
        
        return

    def item_sethide(self, parent, item):
        mode = self.tree.getmode(parent)
        if (mode == "open"):
            self.tree.hlist.hide_entry(item)
        elif (mode == "close"):
            pass
        else:
            self.tree.setmode(parent, "open")
            self.tree.hlist.hide_entry(item)
            pass
        return

    def prepend_item(self, o, name, value, data=None):
        if (self.in_destroy):
            return
        item = o.treeroot + '.' + str(self.itemval)
        if (data == None):
            data = IPMITreeDummyItem(item)
            pass
        data.name_str = name
        self.itemval += 1
        self.tree.hlist.add(item, itemtype=Tix.TEXT, text=name + ":", at=0)
        mode = self.tree.getmode(o.treeroot)
        self.item_sethide(o.treeroot, item)
        if (value == None):
            self.tree.hlist.item_create(item, 1, itemtype=Tix.TEXT, text="",
                                        style=self.active_style)
            self.tree.hlist.item_configure(item, 0, style=self.inactive_style)
        else:
            self.tree.hlist.item_create(item, 1, itemtype=Tix.TEXT, text=value,
                                        style=self.active_style)
            self.tree.hlist.item_configure(item, 0, style=self.active_style)
            pass
        self.treedata[item] = data
        return item

    def append_item(self, o, name, value, data=None, parent=None):
        if (self.in_destroy):
            return
        if (parent == None):
            parent = o.treeroot
            pass
        item = parent + '.' + str(self.itemval)
        if (data == None):
            data = IPMITreeDummyItem(item)
            pass
        data.name_str = name
        self.itemval += 1
        self.tree.hlist.add(item, itemtype=Tix.TEXT, text=name + ":")
        mode = self.tree.getmode(parent)
        if (mode == "open"):
            self.tree.hlist.hide_entry(item)
        elif (mode == "close"):
            pass
        else:
            self.tree.setmode(parent, "open")
            self.tree.hlist.hide_entry(item)
            pass
        if (value == None):
            self.tree.hlist.item_create(item, 1, itemtype=Tix.TEXT, text="",
                                        style=self.active_style)
            self.tree.hlist.item_configure(item, 0, style=self.inactive_style)
        else:
            self.tree.hlist.item_create(item, 1, itemtype=Tix.TEXT, text=value,
                                        style=self.active_style)
            self.tree.hlist.item_configure(item, 0, style=self.active_style)
            pass
        self.treedata[item] = data
        return item

    def set_item_text(self, item, value):
        if (self.in_destroy):
            return
        data = self.treedata[item]
        data.itemvalue = value
        if (hasattr(data, "impt_data") and (data.impt_data != None)):
            self.set_impt_data_text(data)
            pass
        if (value == None):
            self.tree.hlist.item_configure(item, 1, text="")
            self.tree.hlist.item_configure(item, 0, style=self.inactive_style)
            pass
        else:
            self.tree.hlist.item_configure(item, 1, text=value)
            if (hasattr(data, "active")):
                if (data.active):
                    self.set_item_color(item)
                    pass
                pass
            else:
                self.tree.hlist.item_configure(item, 0, style=self.active_style)
                pass
            pass
        return

    def set_item_inactive(self, item):
        if (self.in_destroy):
            return
        data = self.treedata[item]
        data.active = False
        if (hasattr(data, "impt_data") and (data.impt_data != None)):
            self.set_impt_active_change(data)
            pass
        self.tree.hlist.item_configure(item, 0, style=self.inactive_style)
        return

    def set_item_active(self, item):
        if (self.in_destroy):
            return
        data = self.treedata[item]
        data.active = True
        self.set_item_color(item)
        return

    def parent_item(self, item):
        idx = item.rfind(".")
        if (idx == -1):
            return None
        return item[0:idx]
        
    def set_item_color(self, item):
        data = self.treedata[item]
        if (data.num_critical > 0):
            style = self.critical_style
        elif (data.num_severe > 0):
            style=self.severe_style
        elif (data.num_warning > 0):
            style=self.warn_style
        else:
            style=self.active_style
            pass
        if (hasattr(data, "impt_data") and (data.impt_data != None)):
            self.set_impt_style(data, style)
            pass
        self.tree.hlist.item_configure(item, 0, style=style)
        return
        
    def incr_item_warning(self, item):
        if (self.in_destroy):
            return
        data = self.treedata[item]
        if (data == None):
            return
        parent = self.parent_item(item)
        if (parent != None):
           self.incr_item_warning(parent); 
           pass
        data.num_warning = data.num_warning + 1
        if (not data.active):
            return
        if (data.num_critical > 0):
            return
        if (data.num_severe > 0):
            return
        if (data.num_warning == 1):
            self.tree.hlist.item_configure(item, 0, style=self.warn_style)
            pass
        return
        
    def decr_item_warning(self, item):
        if (self.in_destroy):
            return
        data = self.treedata[item]
        if (data == None):
            return
        parent = self.parent_item(item)
        if (parent != None):
           self.decr_item_warning(parent); 
           pass
        data.num_warning = data.num_warning - 1
        if (not data.active):
            return
        if (data.num_critical > 0):
            return
        if (data.num_severe > 0):
            return
        if (data.num_warning > 0):
            return
        self.tree.hlist.item_configure(item, 0, style=self.active_style)
        return
        
    def incr_item_severe(self, item):
        if (self.in_destroy):
            return
        data = self.treedata[item]
        if (data == None):
            return
        parent = self.parent_item(item)
        if (parent != None):
           self.incr_item_severe(parent); 
           pass
        data.num_severe = data.num_severe + 1
        if (not data.active):
            return
        if (data.num_critical > 0):
            return
        if (data.num_severe == 1):
            self.tree.hlist.item_configure(item, 0, style=self.severe_style)
            pass
        return
        
    def decr_item_severe(self, item):
        if (self.in_destroy):
            return
        data = self.treedata[item]
        if (data == None):
            return
        parent = self.parent_item(item)
        if (parent != None):
           self.decr_item_severe(parent); 
           pass
        data.num_severe = data.num_severe - 1
        if (not data.active):
            return
        if (data.num_critical > 0):
            return
        if (data.num_severe > 0):
            return
        if (data.num_warning > 0):
            self.tree.hlist.item_configure(item, 0, style=self.warn_style)
            return
        self.tree.hlist.item_configure(item, 0, style=self.active_style)
        return
        
    def incr_item_critical(self, item):
        if (self.in_destroy):
            return
        data = self.treedata[item]
        if (data == None):
            return
        parent = self.parent_item(item)
        if (parent != None):
           self.incr_item_critical(parent); 
           pass
        data.num_critical = data.num_critical + 1
        if (not data.active):
            return
        if (data.num_critical == 1):
            self.tree.hlist.item_configure(item, 0, style=self.critical_style)
            pass
        return
        
    def decr_item_critical(self, item):
        if (self.in_destroy):
            return
        data = self.treedata[item]
        if (data == None):
            return
        parent = self.parent_item(item)
        if (parent != None):
           self.decr_item_critical(parent); 
           pass
        data.num_critical = data.num_critical - 1
        if (not data.active):
            return
        if (data.num_critical > 0):
            return
        if (data.num_severe > 0):
            self.tree.hlist.item_configure(item, 0, style=self.severe_style)
            return
        if (data.num_warning > 0):
            self.tree.hlist.item_configure(item, 0, style=self.warn_style)
            return
        self.tree.hlist.item_configure(item, 0, style=self.active_style)
        return
        
    def TreeMenu(self, event):
        w = event.widget
        item = w.nearest(event.y)
        data = self.treedata[item]
        if (data != None) and (hasattr(data, "HandleMenu")):
            data.HandleMenu(event)
            pass
        return

    def TreeExpanded(self, event):
        item = event.GetItem()
        data = self.tree.GetPyData(item)
        if (data != None) and (hasattr(data, "HandleExpand")):
            data.HandleExpand(event)
            pass
        return

    def remove_domain(self, d):
        if (self.in_destroy):
            return
        if (hasattr(d, "treeroot")):
            self.cleanup_item(d.treeroot)
            self.tree.hlist.delete_entry(d.treeroot)
            pass
        return

    def add_connection(self, d, c):
        if (self.in_destroy):
            return
        parent = d.treeroot + ".C"
        item = parent + '.' + str(self.itemval)
        self.itemval += 1
        c.treeroot = item
        c.name_str = str(c)
        self.tree.hlist.add(item, itemtype=Tix.TEXT, text=c.name_str)
        self.tree.setmode(item, "none")
        self.tree.close(item)
        self.item_sethide(parent, item)
        self.treedata[item] = c
        self.setup_item(item, active=True)
        return
        
    def add_port(self, c, p):
        if (self.in_destroy):
            return
        item = c.treeroot + '.' + str(self.itemval)
        self.itemval += 1
        p.treeroot = item
        p.name_str = str(p)
        self.tree.hlist.add(item, itemtype=Tix.TEXT, text=p.name_str)
        self.tree.setmode(item, "none")
        self.tree.close(item)
        self.item_sethide(c.treeroot, item)
        self.treedata[item] = p
        self.setup_item(item, active=True)
        return
        
    def remove_port(self, p):
        if (self.in_destroy):
            return
        if (hasattr(p, "treeroot")):
            self.cleanup_item(p.treeroot)
            self.tree.hlist.delete_entry(p.treeroot)
            del self.treedata[p.treeroot]
            pass
        return

    def add_entity(self, d, e, parent=None):
        if (self.in_destroy):
            return
        if (parent == None):
            parent = d.treeroot + ".E"
            pass
        else:
            parent = parent.treeroot
            pass
        e.name_str = str(e)
        item = parent + '.' + str(self.itemval)
        self.itemval += 1
        e.treeroot = item
        self.tree.hlist.add(item, itemtype=Tix.TEXT, text=e.name_str)
        self.tree.setmode(item, "open")
        self.tree.close(item)
        self.item_sethide(parent, item)
        self.treedata[item] = e
        self.setup_item(item, type="entity")

        lstr = item + ".S"
        self.tree.hlist.add(lstr, itemtype=Tix.TEXT, text="Sensors")
        self.tree.setmode(lstr, "none")
        self.tree.close(lstr)
        self.tree.hlist.hide_entry(lstr)
        self.treedata[lstr] = IPMITreeDummyItem(lstr)
        self.setup_item(lstr, active=True)

        lstr = item + ".C"
        self.tree.hlist.add(lstr, itemtype=Tix.TEXT, text="Controls")
        self.tree.setmode(lstr, "none")
        self.tree.close(lstr)
        self.tree.hlist.hide_entry(lstr)
        self.treedata[lstr] = IPMITreeDummyItem(lstr)
        self.setup_item(lstr, active=True)
        return

    def reparent_entity(self, d, e, parent):
        if (self.in_destroy):
            return
        self.add_entity(d, e, parent)
        return
    
    def remove_entity(self, e):
        if (self.in_destroy):
            return
        if (hasattr(e, "treeroot")):
            self.cleanup_item(e.treeroot)
            self.tree.hlist.delete_entry(e.treeroot)
            del self.treedata[e.treeroot]
            pass
        return

    def add_mc(self, d, m):
        if (self.in_destroy):
            return
        parent = d.treeroot + ".M"
        m.name_str = str(m)
        item = parent + "." + str(self.itemval)
        self.itemval += 1
        m.treeroot = item
        self.tree.hlist.add(item, itemtype=Tix.TEXT, text=m.name_str)
        self.tree.setmode(item, "none")
        self.tree.close(item)
        self.item_sethide(parent, item)
        self.treedata[item] = m
        self.setup_item(item)
        return

    def remove_mc(self, m):
        if (self.in_destroy):
            return
        if (hasattr(m, "treeroot")):
            self.cleanup_item(m.treeroot)
            self.tree.hlist.delete_entry(m.treeroot)
            del self.treedata[m.treeroot]
            pass
        return

    def add_sensor(self, e, s):
        if (self.in_destroy):
            return
        parent = e.treeroot + ".S"
        s.name_str = str(s)
        item = parent + "." + str(self.itemval)
        self.itemval += 1
        s.treeroot = item
        self.tree.hlist.add(item, itemtype=Tix.TEXT, text=s.name_str)
        self.tree.setmode(item, "none")
        self.tree.close(item)
        self.item_sethide(parent, item)
        self.treedata[item] = s
        self.setup_item(item, active=True, type="sensor")
        return

    def remove_sensor(self, s):
        if (self.in_destroy):
            return
        if (hasattr(s, "treeroot")):
            self.cleanup_item(s.treeroot)
            self.tree.hlist.delete_entry(s.treeroot)
            del self.treedata[s.treeroot]
            pass
        return

    def add_control(self, e, c):
        if (self.in_destroy):
            return
        parent = e.treeroot + ".C"
        c.name_str = str(c)
        item =  parent + "." + str(self.itemval)
        self.itemval += 1
        c.treeroot = item
        self.tree.hlist.add(item, itemtype=Tix.TEXT, text=c.name_str)
        self.tree.setmode(item, "none")
        self.tree.close(item)
        self.item_sethide(parent, item)
        self.treedata[item] = c
        self.setup_item(item, active=True, type="control")
        return

    def remove_control(self, c):
        if (self.in_destroy):
            return
        if (hasattr(c, "treeroot")):
            self.cleanup_item(c.treeroot)
            self.tree.hlist.delete_entry(c.treeroot)
            del self.treedata[c.treeroot]
            pass
        return

    # XML preferences handling
    def getTag(self):
        return "guiparms"

    def SaveInfo(self, doc, elem):
        elem.setAttribute("windowwidth", str(self.vpane.winfo_width()))
        elem.setAttribute("windowheight", str(self.vpane.winfo_height()))
        spos = int(self.vpane.panecget("objectsevents", "-size"))
        ipos = int(self.vpane.panecget("importantobjects", "-size"))
        elem.setAttribute("sashposition", str(spos))
        elem.setAttribute("isashposition", str(spos + ipos))
        elem.setAttribute("bsashposition",
                          str(self.hpane.panecget("objects", "-size")))
        #elem.setAttribute("treenamewidth", str(self.tree.GetColumnWidth(0)))
        elem.setAttribute("logevents", str(self.logevents))
        elem.setAttribute("fullevents", str(self.fulleventsv != 0))
        for i in self.impt_objs.values():
            for j in i.values():
                o = doc.createElement("watch")
                o.setAttribute("type", j.type)
                o.setAttribute("name", j.name)
                elem.appendChild(o)
                pass
            pass
        return
    pass

def GetAttrInt(attr, default):
    try:
        return int(attr.nodeValue)
    except Exception, e:
        _oi_logging.error("Error getting init parm " + attr.nodeName +
                          ": " + str(e))
        return default

def GetAttrBool(attr, default):
    if (attr.nodeValue.lower() == "true") or (attr.nodeValue == "1"):
        return True
    elif (attr.nodeValue.lower() == "false") or (attr.nodeValue == "0"):
        return False
    else:
        _oi_logging.error ("Error getting init parm " + attr.nodeName)
        pass
    return default

class _GUIRestore(_saveprefs.RestoreHandler):
    def __init__(self):
        _saveprefs.RestoreHandler.__init__(self, "guiparms")
        return

    def restore(self, node):
        global init_windowheight
        global init_windowwidth
        global init_sashposition
        global init_bsashposition
        global init_isashposition
        global init_treenamewidth
        global init_fullevents
        global init_logevents
        global init_impt_objs
        
        for i in range(0, node.attributes.length):
            attr = node.attributes.item(i)
            if (attr.nodeName == "windowwidth"):
                init_windowwidth = GetAttrInt(attr, init_windowwidth)
            elif (attr.nodeName == "windowheight"):
                init_windowheight = GetAttrInt(attr, init_windowheight)
            elif (attr.nodeName == "sashposition"):
                init_sashposition = GetAttrInt(attr, init_sashposition)
            elif (attr.nodeName == "bsashposition"):
                init_bsashposition = GetAttrInt(attr, init_bsashposition)
            elif (attr.nodeName == "isashposition"):
                init_isashposition = GetAttrInt(attr, init_isashposition)
            elif (attr.nodeName == "treenamewidth"):
                init_treenamewidth = GetAttrInt(attr, init_treenamewidth)
            elif (attr.nodeName == "logevents"):
                init_logevents = GetAttrBool(attr, init_logevents)
            elif (attr.nodeName == "fullevents"):
                init_fullevents = GetAttrBool(attr, init_fullevents)
                pass
            pass
        for i in node.childNodes:
            if (i.nodeName == "watch"):
                name = None
                type = None
                for j in range(0, i.attributes.length):
                    attr = i.attributes.item(j)
                    if (attr.nodeName == "name"):
                        name = attr.nodeValue
                    elif (attr.nodeName == "type"):
                        type = attr.nodeValue
                    pass
                if (name != None) and (type != None):
                    init_impt_objs.append( (type, name) )
                    pass
                pass
            pass
        return
    
    pass

_GUIRestore()
    



# _mc.py
#
# openipmi GUI handling for MCs
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2005 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
import OpenIPMI
import gui_popup
import gui_setdialog
import _oi_logging
import _sel
import _mc_chan
import _mc_pefparm

class MCOpHandler:
    def __init__(self, m, func, handler=None, boolval=None):
        self.m = m
        self.func = func
        self.handler = handler
        self.boolval = boolval
        self.item = None
        pass

    def SetItem(self, item):
        self.item = item
        return

    def DoOp(self):
        if (self.boolval):
            if (not getattr(self.m, self.boolval)):
                if (self.item):
                    self.m.ui.set_item_text(self.item, None)
                    pass
                return 0
            pass
        rv = self.m.mc_id.to_mc(self)
        if (rv == 0):
            rv = self.rv
            pass
        return

    def DoUpdate(self):
        if (self.boolval):
            if (not getattr(self.m, self.boolval)):
                if (self.item):
                    self.m.ui.set_item_text(self.item, None)
                    pass
                return
        self.m.mc_id.to_mc(self)
        return

    def mc_cb(self, mc):
        self.rv = getattr(mc, self.func)(self.handler)
        return

    pass

class MCRefreshData:
    def __init__(self, m, func):
        self.m = m;
        self.item = None
        self.func = func
        return

    def SetItem(self, item):
        self.item = item
        return

    def DoUpdate(self):
        if (not self.item):
            return
        if (not self.m.mc_id):
            return
        self.m.mc_id.to_mc(self)
        return

    def mc_cb(self, mc):
        val = getattr(mc, self.func)()
        self.m.ui.set_item_text(self.item, str(val))
        return

    pass

class MCSelSet:
    def __init__(self, m):
        self.m = m;
        self.refr = MCRefreshData(m, "get_sel_rescan_time")
        return

    def DoUpdate(self):
        self.refr.DoUpdate()
        return

    def SetItem(self, item):
        self.refr.SetItem(item)
        return

    def HandleMenu(self, event):
        gui_popup.popup(self.m.ui, event,
                        [ ("Modify Value", self.modval) ])
        return

    def modval(self, event):
        self.init = True
        self.m.mc_id.to_mc(self)

        gui_setdialog.SetDialog("Set SEL Rescan Time for " + str(self.m),
                                [ str(self.sel_rescan_time) ],
                                1,
                                self)
        return

    def ok(self, vals):
        self.ival = int(vals[0])
        self.init = False
        rv = self.m.mc_id.to_mc(self)
        if (rv == 0):
            rv = self.err
            pass
        if (rv != 0):
            return ("Error setting SEL rescan time: "
                    + OpenIPMI.get_error_string(rv))
        return

    def mc_cb(self, mc):
        if self.init:
            self.sel_rescan_time = mc.get_sel_rescan_time()
        else:
            self.err = mc.set_sel_rescan_time(self.ival)
            self.refr.DoUpdate()
            pass
        return

    pass

class PEFLockClearer:
    def __init__(self, mc):
        self.pef = mc.get_pef(self)
        return

    def got_pef_cb(self, pef, err):
        pef.clear_lock();
        return

    pass
        
class MC:
    def __init__(self, d, mc):
        self.d = d
        self.mc_id = mc.get_id()
        self.name = mc.get_name()
        self.cb_state = ""
        d.mcs[self.name] = self
        self.ui = d.ui;
        self.ui.add_mc(self.d, self)
        mc.add_active_handler(self)
        self.has_sel = (mc.sel_device_support() != 0)
        self.event_gen = (mc.ipmb_event_generator_support() != 0)
        self.has_dev_sdrs = (mc.provides_device_sdrs() != 0)
        self.provides_device_sdrs = self.ui.append_item(self,
                                          "Provides Device SDRs",
                                          str(mc.provides_device_sdrs() != 0))
        self.device_available = self.ui.append_item(self, "Device Available",
                                          str(mc.device_available() != 0))
        self.chassis_support = self.ui.append_item(self, "Chassis Support",
                                          str(mc.chassis_support() != 0))
        self.bridge_support = self.ui.append_item(self, "Bridge Support",
                                          str(mc.bridge_support() != 0))
        self.ipmb_event_generator_support = self.ui.append_item(self,
                                   "IPMB Event Generator Support",
                                   str(mc.ipmb_event_generator_support() != 0))
        self.ipmb_event_receiver_support = self.ui.append_item(self,
                                   "IPMB Event Receiver Support",
                                   str(mc.ipmb_event_receiver_support() != 0))
        self.fru_inventory_support = self.ui.append_item(self,
                                   "FRU Inventory Support",
                                   str(mc.fru_inventory_support() != 0))
        self.sel_device_support = self.ui.append_item(self,
                                   "SEL Device Support",
                                   str(mc.sel_device_support() != 0))
        self.sdr_repository_support = self.ui.append_item(self,
                                   "SDR Repository Support",
                                   str(mc.sdr_repository_support() != 0))
        self.sensor_device_support = self.ui.append_item(self,
                                   "Sensor Device Support",
                                   str(mc.sensor_device_support() != 0))
        self.device_id = self.ui.append_item(self, "Device ID",
                                             str(mc.device_id()))
        self.device_revision = self.ui.append_item(self, "Device Revision",
                                                   str(mc.device_revision()))
        self.fw_revision = self.ui.append_item(self, "Firmware Revisions",
                                               str(mc.major_fw_revision())
                                               + "."
                                               + str(mc.minor_fw_revision()))
        self.version = self.ui.append_item(self, "IPMI Version",
                                           str(mc.major_version())
                                           + "."
                                           + str(mc.minor_version()))
        self.manufacturer_id = self.ui.append_item(self,
                                                   "Manufacturer ID",
                                                   str(mc.manufacturer_id()))
        self.product_id = self.ui.append_item(self, "Product ID",
                                              str(mc.product_id()))
        self.aux_fw_revision = self.ui.append_item(self,
                                                   "Aux Firmware Revision",
                                                   mc.aux_fw_revision())
                                               
        self.mguid = self.ui.append_item(self, "GUID", mc.get_guid())

        self.refreshers = [ ]
        self.el_refr = MCOpHandler(self, "get_event_log_enable", self,
                                   "has_sel")
        self.el_item = self.ui.prepend_item(self, "Event Log Enabled",
                                            None, self.el_refr)
        self.refreshers.append(self.el_refr)
        self.ee_refr = self.add_refr_item("Events Enabled",
                                 MCRefreshData(self, "get_events_enable"))
        self.add_refr_item("SEL Rescan Time", MCSelSet(self))

        # Check for PEF capability, send a Get PEF Capabilities cmd
        self.has_pef = False
        mc.send_command(0, 4, 0x10, [ ], self)
        return

    def mc_cmd_cb(self, mc, netfn, cmd, rsp):
        if (rsp[0] != 0):
            # Error
            return
        if (len(rsp) < 4):
            # Response too small
            return
        self.has_pef = True
        return

    def __str__(self):
        return self.name
    
    def add_refr_item(self, name, refr):
        item = self.ui.prepend_item(self, name, None, refr)
        refr.SetItem(item)
        self.refreshers.append(refr)
        return refr
        
    def remove(self):
        self.d.mcs.pop(self.name)
        self.ui.remove_mc(self)
        self.d = None
        self.ui = None
        self.el_refr = None
        self.el_item = None
        self.ee_refr = None
        return

    def Changed(self, mc):
        self.has_sel = (mc.sel_device_support() != 0)
        self.event_gen = (mc.ipmb_event_generator_support() != 0)
        self.has_dev_sdrs = (mc.provides_device_sdrs() != 0)
        self.ui.set_item_text(self.provides_device_sdrs,
                              str(mc.provides_device_sdrs() != 0))
        self.ui.set_item_text(self.device_available,
                              str(mc.device_available() != 0))
        self.ui.set_item_text(self.chassis_support,
                              str(mc.chassis_support() != 0))
        self.ui.set_item_text(self.bridge_support,
                              str(mc.bridge_support() != 0))
        self.ui.set_item_text(self.ipmb_event_generator_support,
                                   str(mc.ipmb_event_generator_support() != 0))
        self.ui.set_item_text(self.ipmb_event_receiver_support,
                                   str(mc.ipmb_event_receiver_support() != 0))
        self.ui.set_item_text(self.fru_inventory_support,
                                   str(mc.fru_inventory_support() != 0))
        self.ui.set_item_text(self.sel_device_support,
                                   str(mc.sel_device_support() != 0))
        self.ui.set_item_text(self.sdr_repository_support,
                                   str(mc.sdr_repository_support() != 0))
        self.ui.set_item_text(self.sensor_device_support,
                                   str(mc.sensor_device_support() != 0))
        self.ui.set_item_text(self.device_id, str(mc.device_id()))
        self.ui.set_item_text(self.device_revision,
                              str(mc.device_revision()))
        self.ui.set_item_text(self.fw_revision,
                              str(mc.major_fw_revision())
                              + "."
                              + str(mc.minor_fw_revision()))
        self.ui.set_item_text(self.version,
                              str(mc.major_version())
                              + "."
                              + str(mc.minor_version()))
        self.ui.set_item_text(self.manufacturer_id, str(mc.manufacturer_id()))
        self.ui.set_item_text(self.product_id, str(mc.product_id()))
        self.ui.set_item_text(self.aux_fw_revision, mc.aux_fw_revision())
                                               
        self.ui.set_item_text(self.mguid, mc.get_guid())
        return

    def HandleExpand(self, event):
        for i in self.refreshers:
            i.DoUpdate()
            pass
        return

    def HandleMenu(self, event):
        l = [ ]
        if self.has_sel:
            l.append( ("Reread SELs", self.RereadSelsHandler) )
            l.append( ("Display SELs", self.DisplaySelsHandler) )
            l.append( ("Enable Event Log", self.EnableEventLogHandler) )
            l.append( ("Disable Event Log", self.DisableEventLogHandler) )
            pass
        if self.event_gen:
            l.append( ("Enable Events", self.EnableEventsHandler) )
            l.append( ("Disable Events", self.DisableEventsHandler) )
            pass
        if self.has_dev_sdrs:
            l.append( ("Refetch SDRs", self.RefetchSDRsHandler) )
            pass
        if self.has_pef:
            l.append( ("PEF Parms", self.PEFParms) )
            l.append( ("Clear PEF Lock", self.PEFLockClear) )
            pass
        l.append( ("Cold Reset", self.ColdResetHandler) )
        l.append( ("Warm Reset", self.WarmResetHandler) )
        l.append( ("Channel Info", self.ChannelInfoHandler) )
        gui_popup.popup(self.ui, event, l)
        return

    def RereadSelsHandler(self, event):
        dop = MCOpHandler(self, "reread_sel")
        dop.DoOp()
        return

    def DisplaySelsHandler(self, event):
        _sel.MCSELDisplay(self.mc_id)
        return

    def EnableEventLogHandler(self, event):
        self.cb_state = "enable_event_log"
        self.mc_id.to_mc(self)
        return

    def DisableEventLogHandler(self, event):
        self.cb_state = "disable_event_log"
        self.mc_id.to_mc(self)
        return

    def EnableEventsHandler(self, event):
        self.cb_state = "enable_events"
        self.mc_id.to_mc(self)
        return

    def DisableEventsHandler(self, event):
        self.cb_state = "disable_events"
        self.mc_id.to_mc(self)
        return

    def ColdResetHandler(self, event):
        self.cb_state = "cold_reset"
        self.mc_id.to_mc(self)
        return

    def WarmResetHandler(self, event):
        self.cb_state = "warm_reset"
        self.mc_id.to_mc(self)
        return

    def ChannelInfoHandler(self, event):
        self.cb_state = "channel_info"
        self.mc_id.to_mc(self)
        return

    def RefetchSDRsHandler(self, event):
        self.cb_state = "refetch_sdrs"
        self.mc_id.to_mc(self)
        return

    def PEFParms(self, event):
        self.cb_state = "pef_parms"
        self.mc_id.to_mc(self)
        return

    def PEFLockClear(self, event):
        self.cb_state = "pef_lock_clear"
        self.mc_id.to_mc(self)
        return

    def mc_cb(self, mc):
        if (self.cb_state == "enable_events"):
            mc.set_events_enable(1, self)
        elif (self.cb_state == "disable_events"):
            mc.set_events_enable(0, self)
        if (self.cb_state == "enable_event_log"):
            mc.set_event_log_enable(1, self)
        elif (self.cb_state == "disable_event_log"):
            mc.set_event_log_enable(0, self)
        elif (self.cb_state == "cold_reset"):
            mc.reset(OpenIPMI.MC_RESET_COLD)
        elif (self.cb_state == "warm_reset"):
            mc.reset(OpenIPMI.MC_RESET_WARM)
        elif (self.cb_state == "refetch_sdrs"):
            mc.reread_sensors()
        elif (self.cb_state == "reread_sel"):
            mc.reread_sel()
        elif (self.cb_state == "channel_info"):
            _mc_chan.MCChan(self, mc)
        elif (self.cb_state == "pef_parms"):
            pp = mc.get_pef(self)
            if (pp == None):
                self.ui.ReportError("Unable to allocate PEF config");
                return
        elif (self.cb_state == "pef_lock_clear"):
            PEFLockClearer(mc)
            pass
        return

    def got_pef_cb(self, pef, err):
        if (err):
            self.ui.ReportError("Error fetching PEF: " +
                                OpenIPMI.get_error_string(err))
            return
        rv = pef.get_config(self)
        if (rv):
            self.ui.ReportError("Error starting PEF config fetch: " +
                                OpenIPMI.get_error_string(rv))
            pass
        return

    def pef_got_config_cb(self, pef, err, pefconfig):
        if (err):
            if (err == OpenIPMI.eagain):
                self.ui.ReportError("PEF already locked by another user, "
                                    "Try clearing the lock.")
                pass
            else:
                self.ui.ReportError("Error fetching PEF config: " +
                                    OpenIPMI.get_error_string(err))
                pass
            return
        _mc_pefparm.MCPefParm(self, pef, pefconfig)
        return

    def mc_events_enable_cb(self, mc, err):
        if (err):
            _oi_logging.error("Error setting MC events for "
                              + self.name + ": "
                              + OpenIPMI.get_error_string(err))
            return
        self.ee_refr.DoUpdate()

    def mc_get_event_log_enable_cb(self, mc, err, val):
        if (err):
            _oi_logging.error("Error getting MC event log enable for "
                              + self.name + ": "
                              + OpenIPMI.get_error_string(err))
            return
        self.ui.set_item_text(self.el_item, str(val != 0))
        
    def mc_set_event_log_enable_cb(self, mc, err):
        if (err):
            _oi_logging.error("Error setting MC event log enable for"
                              + self.name + ": "
                              + OpenIPMI.get_error_string(err))
            return
        self.el_refr.DoUpdate()
        return

    def mc_active_cb(self, mc, active):
        if (active):
            self.ui.set_item_active(self.treeroot)
        else:
            self.ui.set_item_inactive(self.treeroot)
            pass
        return

    pass



# gui_term.py
#
# openipmi GUI terminal handling for SoL
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2006 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import Tix
import _term

def gpos(x, y):
    return str(y+1) + "." + str(x)
        
class Terminal(_term.TerminalEmulator):
    def __init__(self, parent):
        self.fsize = 8

        _term.TerminalEmulator.__init__(self)

        self.fonts = ("-family terminal -size " + str(self.fsize)
                      + " -weight normal -underline 0",
                      # Bold
                      "-family terminal -size " + str(self.fsize)
                      + " -weight bold -underline 0",
                      # Underline
                      "-family terminal -size " + str(self.fsize)
                      + " -weight normal -underline 1",
                      # Bold Underline
                      "-family terminal -size " + str(self.fsize)
                      + " -weight bold -underline 1")

        self.colors = ("black",
                       "red",
                       "green",
                       "yellow",
                       "blue",
                       "magenta",
                       "cyan",
                       "white")

        self.text = Tix.Text(parent, width=self.width, height=self.height,
                             state="normal", wrap="none", font=self.fonts[0])
        self.text.pack(side=Tix.TOP, fill=Tix.BOTH, expand=1)

        # We create a tag for every possible terminal configuration, based
        # upon an equation using the colors and font type.
        for f in range(0, len(self.fonts)):
            for fg in range(0, len(self.colors)):
                for bg in range(0, len(self.colors)):
                    idx = (f * 8 * 8) + (fg * 8) + bg;
                    self.text.tag_configure(str(idx), font=self.fonts[f],
                                            foreground=self.colors[fg],
                                            background=self.colors[bg])
                    pass
                pass
            pass

        self.default_tag = str(7 * 8) # white forground, black background

        for i in range(1, self.height+1):
            self.text.insert(str(i) + ".1", "%*s\n" % (self.width, ""),
                             self.default_tag)
            pass

        self.handle_cursor()

        self.text.bind("<Key>", self.HandleChar)
        self.text.bind("<Control-Key>", self.HandleControlChar)

        self.text.focus_set()
        return

    def DrawText(self, fg_color, bg_color, flags, x, y, val):
        if (flags & _term.INVERSE):
            tmp = bg_color
            bg_color = fg_color
            fg_color = tmp
            pass
        # FIXME - we don't handle blinking or concealed
        flags &= 3
        tag = str((flags * 8 * 8) + (fg_color * 8) + bg_color);
        self.text.delete(gpos(x, y), gpos(x+len(val), y))
        self.text.insert(gpos(x, y), val, tag)
        return

    def DrawCursor(self, fg_color, bg_color, flags, x, y, val):
        # Draw cursor as inverse, just flip the background and foreground
        self.DrawText(bg_color, fg_color, flags, x, y, val)
        return

    def ScrollLines(self, y1, y2):
        self.text.delete(gpos(0, y1), gpos(0, y1+1))
        if (y2 == self.height-1):
            self.text.insert("end", "\n%*s" % (self.width, ""),
                             self.default_tag)
            pass
        else:
            # Not (y2+1), because we deleted a line
            pos = gpos(0, y2)
            self.text.insert(pos, "%*s\n" % (self.width, ""),
                             self.default_tag)
            pass
        return
    
    def ScrollLinesUp(self, y1, y2):
        self.text.delete(gpos(0, y2), gpos(0, y2+1))
        if (y1 == self.height-1):
            self.text.insert("end", "\n%*s" % (self.width, ""),
                             self.default_tag)
            pass
        else:
            pos = gpos(0, y1)
            self.text.insert(pos, "%*s\n" % (self.width, ""),
                             self.default_tag)
            pass
        return

    def DeleteChars(self, x, y, len):
        pos = gpos(x, y)
        endpos = gpos(x+len, y)
        self.text.delete(pos, endpos)
        self.text.insert(str(y+1) + ".end", "%*s" % (len, ""),
                         self.default_tag)
        return
    
    def InsertChars(self, x, y, len):
        self.text.delete(gpos(self.width-len, y), gpos(self.width, y))
        self.text.insert(gpos(x, y), "%*s" % (len, ""),
                         self.default_tag)
        return
    
    def SendBack(self, data):
        self.HandleTerminalOutput(data)
        return

    def Bell(self):
        return

    def RequestSizeChange(self, w, h):
        return
    
    def HandleChar(self, event):
        key = event.keysym
        if (len(key) == 1):
            s = key
        elif (key == "Return") or (key == "KP_Enter"):
            s = "\x0d"
        elif (key == "Backspace"):
            s = "\x08"
        elif (key == "Up"):
            s = "\x1b[A"
        elif (key == "Down"):
            s = "\x1b[B"
        elif (key == "Right"):
            s = "\x1b[C"
        elif (key == "Left"):
            s = "\x1b[D"
        elif (key == "Next"):
            s = "\x1b[6~"
        elif (key == "Prior"):
            s = "\x1b[5~"
        elif (key == "Insert"):
            s = "\x1b[2~"
        elif (key == "Home"):
            s = "\x1b[OH"
        elif (key == "End"):
            s = "\x1b[OF"
        elif (key == "Delete"):
            s = "\x1b[3~"
        elif (key == "F1"):
            s = "\x1bOP"
        elif (key == "F2"):
            s = "\x1bOQ"
        elif (key == "F3"):
            s = "\x1bOR"
        elif (key == "F4"):
            s = "\x1bOS"
        elif (key == "F5"):
            s = "\x1b[15~"
        elif (key == "F6"):
            s = "\x1b[17~"
        elif (key == "F7"):
            s = "\x1b[18~"
        elif (key == "F8"):
            s = "\x1b[19~"
        elif (key == "F9"):
            s = "\x1b[20~"
        elif (key == "F10"):
            s = "\x1b[21~"
        elif (key == "F11"):
            s = "\x1b[23~"
        elif (key == "F12"):
            s = "\x1b[24~"
        elif (len(event.char) == 1) and (event.char < chr(255)):
            s = event.char

        # Keypad stuff comes after the check because if numlock is off, the
        # event.char will be empty
        elif (key == "KP_Add"):
            s = "\x1bOl"
        elif (key == "KP_Subtract"):
            s = "\x1bOm"
        elif (key == "KP_Delete"):
            s = "\x1bOn"
        elif (key == "KP_Multiply"):
            s = "\x1bOQ"
        elif (key == "KP_Divide"):
            s = "\x1bOR"
        elif (key == "KP_Insert"):
            s = "\x1bOp"
        elif (key == "KP_End"):
            s = "\x1bOq"
        elif (key == "KP_Down"):
            s = "\x1bOr"
        elif (key == "KP_Next"):
            s = "\x1bOs"
        elif (key == "KP_Left"):
            s = "\x1bOt"
        elif (key == "KP_Begin"):
            s = "\x1bOu"
        elif (key == "KP_Right"):
            s = "\x1bOv"
        elif (key == "KP_Home"):
            s = "\x1bOw"
        elif (key == "KP_Up"):
            s = "\x1bOx"
        elif (key == "KP_Prior"):
            s = "\x1bOy"
        else:
            return "break"
        self.HandleTerminalOutput(s)
        return "break"
    
    def HandleControlChar(self, event):
        key = event.keysym
        if (len(key) != 1):
            return
        if ((key >= 'A') and (key <= 'B')): # '@' 'A' .. 'Z'
            s = "%c" % (ord(key)-64)
            pass
        elif ((key >= 'a') and (key <= 'z')): # 'a' .. 'z'
            s = "%c" % (ord(key)-96)
        elif (key < ' '):
            s = key
            pass
        else:
            return "break"
        self.HandleTerminalOutput(s)
        return "break"
    
    pass

import sys
class TestTerm(Terminal):
    def __init__(self, parent):
        Terminal.__init__(self, parent)
        return

    def HandleTerminalOutput(self, s):
        # FIXME: hacks to remove
        sys.stdout.write(s)
        sys.stdout.flush()
        if (s[0] == '\x01'):
            self.ProcessInput("abcdefghijk")
            return
        self.ProcessInput(s)
        return
    pass

class MyApp(Tix.Tk):
    def __init__(self):
        Tix.Tk.__init__(self)
        term = TestTerm(self)
        return
    
    pass

if __name__ == "__main__":
    app = MyApp()
    app.mainloop()





# gui_setdialog.py
#
# openipmi GUI handling for a normal dialog
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2006 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import Tix
import gui_errstr

def isbool(v):
    return type(v) == type(True)

class SetDialog(Tix.Toplevel):
    def __init__(self, name, default, count, handler, labels=None,
                 longtext=False):
        self.handler = handler
        Tix.Toplevel.__init__(self)
        self.title(name)
        self.longtext = longtext

        sw = Tix.ScrolledWindow(self)
        self.values = sw.window
        if (labels == None):
            if (count == 1):
                label = Tix.Label(self.values, text="Value:")
            else:
                label = Tix.Label(self.values, text="Value(s):")
                pass
            label.grid(row=0, column=0, sticky="e")
            self.fields = [ ]
            row = 0
            for i in range(0, count):
                if (isbool(default[i])):
                    field = Tix.BooleanVar()
                    field.set(default[i])
                    w = Tix.Checkbutton(self.values, variable=field)
                    pass
                elif longtext:
                    field = Tix.Text(self.values)
                    field.insert("1.0", str(default[i]))
                    w = field
                else:
                    field = Tix.Entry(self.values)
                    field.insert("0", str(default[i]))
                    w = field
                    pass
                w.grid(row=row, column=1, sticky="ew")
                row += 1
                self.fields.append(field)
                pass
            pass
        else:
            self.fields = [ ]
            row = 0
            for i in range(0, count):
                label = Tix.Label(self.values, text=labels[i])
                label.grid(row=row, column=0)
                if (isbool(default[i])):
                    field = Tix.BooleanVar()
                    field.set(default[i])
                    w = Tix.Checkbutton(self.values, variable=field)
                    pass
                elif longtext:
                    field = Tix.Text(self.values)
                    field.insert("1.0", str(default[i]))
                    w = field
                else:
                    field = Tix.Entry(self.values)
                    field.insert("0", str(default[i]))
                    w = field
                    pass
                self.fields.append(field)
                w.grid(row=row, column=1, sticky="ew")
                row += 1
                pass
            pass

        sw.pack(side=Tix.TOP, fill=Tix.BOTH, expand=1)
        
        self.errstr = gui_errstr.ErrStr(self)
        self.errstr.pack(side=Tix.TOP, fill=Tix.X, expand=1)

        bbox = Tix.ButtonBox(self)
        bbox.add("cancel", text="Cancel", command=lambda w=self: w.cancel())
        bbox.add("ok", text="Ok", command=lambda w=self: w.ok())
        bbox.pack(side=Tix.BOTTOM, fill=Tix.X, expand=1)

        self.bind("<Destroy>", self.OnDestroy)
        return
    
    def OnDestroy(self, event):
        if (hasattr(self, "do_on_close")):
            self.do_on_close()
            pass
        self.handler = None
        return

    def cancel(self):
        self.destroy()
        return

    def ok(self):
        vals = [ ]
        for f in self.fields:
            if (self.longtext):
                v = f.get("1.0", "end")
                pass
            else:
                v = f.get()
                pass
            if (not isbool(v)):
                v = v.strip()
                pass
            vals.append(v)
            pass
        try:
            err = self.handler.ok(vals)
            if (err != None):
                self.errstr.SetError(err)
                return
            pass
        except:
            self.errstr.SetError("Value invalid")
            return
        self.destroy()
        return

    pass



# _control.py
#
# openipmi GUI handling for controls
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2005 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import OpenIPMI
import gui_popup
import gui_setdialog
import gui_lightset

class ControlRefreshData:
    def __init__(self, c):
        self.c = c
        return

    def control_cb(self, control):
        if (self.c.control_type == OpenIPMI.CONTROL_IDENTIFIER):
            control.identifier_get_val(self.c)
        elif (self.c.setting_light):
            control.get_light(self.c)
        else:
            control.get_val(self.c)
            pass
        return
    
    pass

class ControlSet:
    def __init__(self, c):
        self.c = c;
        return

    def HandleMenu(self, event):
        if (self.c.impt_data == None):
            ml = [ [ "Add to watch values", self.c.add_impt ] ]
            pass
        else:
            ml = [ [ "Remove from watch values", self.c.remove_impt ] ]
            pass
        ml.append([ "Modify Value", self.modval ])
        ml.append([ "Set to 0",     self.SetTo0 ])
        ml.append([ "Set to 1",     self.SetTo1 ])
        gui_popup.popup(self.c.ui, event, ml)
        return

    def modval(self, event):
        vals = self.c.vals
        while (len(vals) < self.c.num_vals):
            vals.append(0)
            pass
        gui_setdialog.SetDialog("Set Control Values for " + self.c.name,
                                vals,
                                self.c.num_vals,
                                self)
        return

    def SetTo0(self, event):
        self.ival = [ 0 ]
        self.c.control_id.to_control(self)
        return

    def SetTo1(self, event):
        self.ival = [ 1 ]
        self.c.control_id.to_control(self)
        return

    def ok(self, vals):
        self.ival = [ ]
        for val in vals:
            self.ival.append(int(val))
            pass
        self.c.control_id.to_control(self)
        return

    def control_cb(self, control):
        if (self.c.control_type == OpenIPMI.CONTROL_IDENTIFIER):
            control.identifier_set_val(self.ival)
        else:
            control.set_val(self.ival)
            pass
        return

    def do_on_close(self):
        self.c = None

    pass
        
class LightSet:
    def __init__(self, c):
        self.c = c;
        return

    def HandleMenu(self, event):
        if (self.c.impt_data == None):
            ml = [ [ "Add to watch values", self.c.add_impt ] ]
            pass
        else:
            ml = [ [ "Remove from watch values", self.c.remove_impt ] ]
            pass
        ml.append([ "Modify Value", self.modval ])
        gui_popup.popup(self.c.ui, event, ml)
        return

    def modval(self, event):
        gui_lightset.LightSet("Set Light Values for " + self.c.name,
                              self.c.num_vals, self.c.lights, self.c.vals,
                              self);
        return

    def ok(self, val):
        self.ival = ';'.join(val)
        self.c.control_id.to_control(self)
        return

    def control_cb(self, control):
        rv = control.set_light(self.ival)
        if (rv != 0):
            raise ValueError("set_light failed: " + str(rv))
        return
        
class Control:
    def __init__(self, e, control):
        self.e = e
        self.name = control.get_name()
        e.controls[self.name] = self
        self.control_type_str = control.get_type_string()
        self.control_id = control.get_id()
        self.ui = e.ui;
        self.updater = ControlRefreshData(self)
        self.vals = [ ]
        self.ui.add_control(self.e, self)
        self.control_type = control.get_type()
        self.destroyed = False
        if (self.control_type == OpenIPMI.CONTROL_IDENTIFIER):
            self.num_vals = control.identifier_get_max_length();
        else:
            self.num_vals = control.get_num_vals();
            pass

        if ((self.control_type == OpenIPMI.CONTROL_LIGHT)
            and (control.light_set_with_setting())):
            self.setting_light = True
            self.lights = [ ]
            for  i in range (0, self.num_vals):
                lc = control.light_has_local_control(i)
                colors = [ ]
                for j in range (0, OpenIPMI.CONTROL_NUM_COLORS):
                    if control.light_is_color_supported(i, j):
                        colors.append(OpenIPMI.color_string(j))
                        pass
                    pass
                self.lights.append((lc, colors))
                pass
            pass
        else:
            self.setting_light = False
            pass

        self.is_settable = control.is_settable()
        self.is_readable = control.is_readable()
        if (self.is_settable):
            if (self.setting_light):
                self.setter = LightSet(self)
            else:
                self.setter = ControlSet(self)
                pass
            pass
        else:
            self.setter = None
            pass

        self.ui.prepend_item(self, "Control Type", self.control_type_str)
        cs = [ ]
        if (control.has_events()):
            cs.append("generates_events")
            pass
        if (self.is_settable):
            cs.append("settable")
            pass
        if (self.is_readable):
            cs.append("readable")
            pass
        self.ui.append_item(self, "Control Capabilities", ' '.join(cs))
        if (self.control_type == OpenIPMI.CONTROL_LIGHT):
            self.ui.append_item(self, "Num Lights", str(self.num_vals))
            if (self.setting_light):
                self.ui.append_item(self, "Light Type", "setting")
                for i in range(0, self.num_vals):
                    cap = [ ]
                    if control.light_has_local_control(i):
                        cap.append("local_control")
                        pass
                    for j in range (0, OpenIPMI.CONTROL_NUM_COLORS):
                        if control.light_is_color_supported(i, j):
                            cap.append(OpenIPMI.color_string(j))
                            pass
                        pass
                    self.ui.append_item(self, "Light" + str(i), ' '.join(cap))
                    pass
                pass
            else:
                self.ui.append_item(self, "Light Type", "transition")
                for i in range(0, self.num_vals):
                    cap = [ ]
                    for j in range (0, control.get_num_light_values(i)):
                        cap2 = [ ]
                        for k in range (0,control.get_num_light_transitions(i, j)):
                            cap3 = [ ]
                            val = control.get_light_color(i, j, k)
                            cap3.append(OpenIPMI.color_string(val))
                            val = control.get_light_color_time(i, j, k)
                            cap3.append(OpenIPMI.color_string(val))
                            cap2.append(cap3)
                            pass
                        cap.append(cap2)
                        pass
                    self.ui.append_item(self, "Light" + str(i), str(cap))
                    pass
                pass
            pass
        elif (self.control_type == OpenIPMI.CONTROL_IDENTIFIER):
            self.ui.append_item(self, "Max Length", str(self.num_vals))
        else:
            self.ui.append_item(self, "Num Vals", str(self.num_vals))
            pass
        return
    
    def __str__(self):
        return self.name

    def DoUpdate(self):
        if (self.is_readable):
            self.control_id.to_control(self.updater)
            pass
        return

    def HandleMenu(self, event):
        if (self.setter != None):
            self.setter.HandleMenu(event)
            pass
        else:
            if (self.impt_data == None):
                ml = [ [ "Add to watch values", self.add_impt ] ]
                pass
            else:
                ml = [ [ "Remove from watch values", self.remove_impt ] ]
                pass
            gui_popup.popup(self.ui, event, ml)
            pass
        return

    def add_impt(self, event):
        self.ui.add_impt_data("control", str(self), self)
        return
        
    def remove_impt(self, event):
        self.ui.remove_impt_data(self.impt_data)
        return
        
    def control_get_val_cb(self, control, err, vals):
        if (self.destroyed):
            return
        if (err != 0):
            self.ui.set_item_text(self.treeroot, None)
            return
        self.num_vals = control.get_num_vals();
        self.vals = vals
        self.ui.set_item_text(self.treeroot, str(vals))
        return
        
    def control_get_id_cb(self, control, err, val):
        if (self.destroyed):
            return
        if (err != 0):
            self.ui.set_item_text(self.treeroot, None)
            return
        self.num_vals = control.identifier_get_max_length();
        self.val = val
        self.ui.set_item_text(self.treeroot, str(val))
        return

    def control_get_light_cb(self, control, err, vals):
        if (self.destroyed):
            return
        if (err != 0):
            self.ui.set_item_text(self.treeroot, None)
            return
        self.num_vals = control.get_num_vals();
        v1 = vals.split(":")
        self.vals = [ ]
        for s1 in v1:
            v1 = s1.split()
            if (v1[0] != "lc"):
                v1.insert(0, "")
                pass
            self.vals.append(v1)
            pass
        self.ui.set_item_text(self.treeroot, str(self.vals))
        return

    def remove(self):
        self.e.controls.pop(self.name)
        self.ui.remove_control(self)
        self.destroyed = True
        self.e = None
        self.updater = None
        self.ui = None
        return

    pass



# _domain.py
#
# openipmi GUI handling for domains
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2006 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import Tix

class PopupSelector:
    def __init__(self, handler, val, pd):
        self.handler = handler
        self.val = val;
        self.pd = pd
        return

    def handle(self):
        self.pd.done = True
        self.handler(self.val)
        return

    pass

class PopupDone:
    def __init__(self):
        self.done = False;
        pass

    def setdone(self, event):
        self.done = True
        return
    
    pass

def popup(ui, event, handlers, point=None):
    menu = Tix.Menu(ui, tearoff=0);
    pd = PopupDone()
    for h in handlers:
        if (len(h) >= 3):
            p = PopupSelector(h[1], h[2], pd)
            pass
        else:
            p = PopupSelector(h[1], None, pd)
            pass
        menu.add("command", command=p.handle, label=h[0])
        pass
    if (point == None):
        point = event
        pass
    menu.post(point.x_root, point.y_root)
    menu.grab_set_global()
    menu.bind("<FocusOut>", pd.setdone)
    menu.bind("<ButtonRelease-3>", pd.setdone)
    while (not pd.done):
        event.widget.tk.dooneevent()
        pass
    menu.grab_release()
    menu.destroy()
    return



# _mc_chan.py
#
# openipmi GUI handling for MC channel info
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2005 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import OpenIPMI
import gui_popup
import gui_treelist
import _oi_logging
import _mc_lanparm
import _mc_solparm
import _mc_user

# Collect all the info for the channels of an MC.  For each channel
# all the info is requested using the types immediately below.  If the
# fetch is a success then it is added to the "info" array of the main
# type.  When all the info is collected, then it is all displayed.
#
# The info array is an array of hashes, one element for each channel.
# Each hash containes one of the following elements: "info" for
# channel info, "n" for non-volatile user access, and "v" for volatile
# user access.

class MCChanUserAcc:
    def __init__(self, mcchan, idx, t):
        self.mcchan = mcchan
        self.idx = idx
        self.t = t
        if (t == "v"):
            self.tstr = "volatile"
        else:
            self.tstr = "nonvolatile"
            pass
        return

    def mc_channel_got_access_cb(self, mc, err, access):
        if (self.mcchan.info == None):
            return
        if (not err):
            self.access = access
            self.mcchan.info[self.idx][self.t] = self
            pass
        self.mcchan.done_one()
        return

    pass

class MCChanInfo:
    def __init__(self, mcchan, idx):
        self.mcchan = mcchan
        self.idx = idx
        return

    def mc_channel_got_info_cb(self, mc, err, info):
        if (self.mcchan.info == None):
            return
        if (not err):
            self.info = info;
            self.mcchan.info[self.idx]["info"] = self
            v = [ 0 ]
            rv = info.get_medium(v)
            if ((not rv) and (v[0] == OpenIPMI.CHANNEL_MEDIUM_8023_LAN)):
                # Test for SoL with get channel payload support cmd
                rv = mc.send_command(0, 6, 0x4e, [ self.idx ], self)
                if (not rv):
                    return
                pass
            pass
        self.mcchan.done_one()
        return

    def mc_cmd_cb(self, mc, netfn, cmd, rsp):
        if (rsp[0] != 0):
            # Error
            self.mcchan.done_one()
            return
        if (len(rsp) < 9):
            # Eh?  response is too small
            self.mcchan.done_one()
            return
        if (rsp[1] & 0x2):
            # We have SoL support
            self.mcchan.info[self.idx]["SoL"] = True
            pass
        self.mcchan.done_one()
        return

    pass

class MCChanData:
    def __init__(self, mcchan, idx):
        self.idx = idx;
        self.mcchan = mcchan
        # Assume this unless told otherwise
        self.medium = OpenIPMI.CHANNEL_MEDIUM_IPMB
        self.has_sol_set = False
        return

    def HandleMenu(self, event, eitem, point):
        if (self.mcchan.info == None):
            return
        l = [ ("User Info", self.users) ]
        if (self.medium == OpenIPMI.CHANNEL_MEDIUM_8023_LAN):
            l.append( ("LANPARMS", self.lanparms) )
            l.append( ("Clear LANPARM lock", self.clr_lanparm_lock) )
            if ("SoL" in self.mcchan.info[self.idx]):
                l.append( ("SoLPARMS", self.solparms) )
                l.append( ("Clear SoLPARM lock", self.clr_solparm_lock) )
                pass
            pass
        gui_popup.popup(self.mcchan, event, l, point)
        return
    
    def users(self, event):
        self.cb_state = "users"
        self.mcchan.mc_id.to_mc(self)
        return

    def lanparms(self, event):
        self.cb_state = "lanparms"
        self.mcchan.mc_id.to_mc(self)
        return

    def clr_lanparm_lock(self, event):
        self.cb_state = "clr_lanparm_lock"
        self.mcchan.mc_id.to_mc(self)
        return

    def solparms(self, event):
        self.cb_state = "solparms"
        self.mcchan.mc_id.to_mc(self)
        return

    def clr_solparm_lock(self, event):
        self.cb_state = "clr_solparm_lock"
        self.mcchan.mc_id.to_mc(self)
        return

    def mc_cb(self, mc):
        if (self.cb_state == "users"):
            rv = mc.get_users(self.idx, 0, self)
            if (rv):
                self.mcchan.SetError("Could not get users: " +
                                     OpenIPMI.get_error_string(rv))
                pass
            pass
        elif (self.cb_state == "lanparms"):
            lp = mc.get_lanparm(self.idx)
            lp.get_config(self)
        elif (self.cb_state == "clr_lanparm_lock"):
            lp = mc.get_lanparm(self.idx)
            lp.clear_lock()
        elif (self.cb_state == "solparms"):
            sp = mc.get_solparm(self.idx)
            sp.get_config(self)
        elif (self.cb_state == "clr_solparm_lock"):
            sp = mc.get_solparm(self.idx)
            sp.clear_lock()
            pass
        return

    def mc_channel_got_users_cb(self, mc, err, max_users, enabled_users,
                                fixed_users, users):
        if (err):
            self.mcchan.SetError("Error fetching users: " +
                                 OpenIPMI.get_error_string(err))
            return
        if (len(users) == 0):
            self.mcchan.SetError("No users")
            return
        v = [ 0 ]
        users[0].get_channel(v)
        ch = v[0]
        _mc_user.MCUsers(mc, ch, max_users, enabled_users, fixed_users, users)
        return
    
    def lanparm_got_config_cb(self, lanparm, err, lanconfig):
        if (err):
            if (err == OpenIPMI.eagain):
                self.mcchan.SetError(
                    "LANPARMs are locked, clear the lock if necessary")
            else:
                self.mcchan.SetError(
                    "Error getting lanparms: "
                    + OpenIPMI.get_error_string(err))
                pass
            return
        _mc_lanparm.MCLanParm(self.mcchan.m, lanparm, lanconfig, self.idx)
        return
    
    def solparm_got_config_cb(self, solparm, err, solconfig):
        if (err):
            if (err == OpenIPMI.eagain):
                self.mcchan.SetError(
                    "SOLPARMs are locked, clear the lock if necessary")
            else:
                self.mcchan.SetError(
                    "Error getting solparms: "
                    + OpenIPMI.get_error_string(err))
                pass
            return
        _mc_solparm.MCSolParm(self.mcchan.m, solparm, solconfig, self.idx)
        return
    
    pass

class BoolSetter:
    def __init__(self, mcchan, setter):
        self.mcchan = mcchan
        self.setter = setter
        return
    
    def SetItem(self, item):
        self.item = item
        return
    
    def HandleMenu(self, event, eitem, point):
        gui_popup.popup(self.mcchan, event,
                        [ ("Toggle Value", self.togglevalue) ],
                        point)
        return

    def togglevalue(self, event):
        val = str(self.mcchan.GetColumn(self.item, 1))
        if (val == "True") or (val == "true"):
            val = "false"
            bval = 0
        else:
            val = "true"
            bval = 1
            pass
        rv = self.setter(bval)
        if (rv):
            self.mcchan.SetError("Could not toggle value: "
                                 + OpenIPMI.get_error_string(rv))
            return
        self.mcchan.SetColumn(self.item, val, 1);
        return

    pass

class AccessSetter:
    def __init__(self, mcchan, setter):
        self.mcchan = mcchan
        self.setter = setter
        return
    
    def SetItem(self, item):
        self.item = item
        return
    
    def HandleMenu(self, event, eitem, point):
        gui_popup.popup(self.mcchan, event,
                        [ ("Disabled", self.disabled),
                          ("PreBoot", self.preboot),
                          ("Always", self.always),
                          ("Shared", self.shared) ],
                        point)
        return

    def setval(self, val):
        rv = self.setter(val)
        if (rv):
            mcchan.SetError("Could not set value: "
                            + OpenIPMI.get_error_string(rv))
            return
        self.mcchan.SetColumn(self.item, 
                              OpenIPMI.channel_access_mode_string(val),
                              1)
        return
        
    def disabled(self, event):
        self.setval(OpenIPMI.CHANNEL_ACCESS_MODE_DISABLED)
        return

    def preboot(self, event):
        self.setval(OpenIPMI.CHANNEL_ACCESS_MODE_PRE_BOOT)
        return

    def always(self, event):
        self.setval(OpenIPMI.CHANNEL_ACCESS_MODE_ALWAYS)
        return

    def shared(self, event):
        self.setval(OpenIPMI.CHANNEL_ACCESS_MODE_SHARED)
        return

    pass

class PrivSetter:
    def __init__(self, mcchan, setter):
        self.mcchan = mcchan
        self.setter = setter
        return

    def SetItem(self, item):
        self.item = item
        return
    
    def HandleMenu(self, event, eitem, point):
        gui_popup.popup(self.mcchan, event,
                        [ ("Callback", self.callback),
                          ("User", self.user),
                          ("Operator", self.operator),
                          ("Admin", self.admin),
                          ("OEM", self.oem) ],
                        point)
        return

    def setval(self, val):
        rv = self.setter(val)
        if (rv):
            mcchan.SetError("Could not set value: "
                            + OpenIPMI.get_error_string(rv))
            return
        self.mcchanSetColumn(self.item, 
                             OpenIPMI.privilege_string(val),
                             1)
        return
        
    def callback(self, event):
        self.setval(OpenIPMI.PRIVILEGE_CALLBACK)
        return

    def user(self, event):
        self.setval(OpenIPMI.PRIVILEGE_USER)
        return

    def operator(self, event):
        self.setval(OpenIPMI.PRIVILEGE_OPERATOR)
        return

    def admin(self, event):
        self.setval(OpenIPMI.PRIVILEGE_ADMIN)
        return

    def oem(self, event):
        self.setval(OpenIPMI.PRIVILEGE_OEM)
        return

    pass

class MCChan(gui_treelist.TreeList):
    def __init__(self, m, mc):
        gui_treelist.TreeList.__init__(self, "Channel info for " + m.name,
                                       "Channels",
                                       [ ("Name", 200), ("Value", 400) ] )
        self.m = m;
        self.mc_id = mc.get_id()
        self.count = 0;
        self.info = []
        for i in range(0, OpenIPMI.MAX_USED_CHANNELS):
            self.info.append({})
            rv = mc.channel_get_info(i, MCChanInfo(self, i))
            if (not rv):
                self.count += 1
                pass
            rv = mc.channel_get_access(i, "volatile",
                                       MCChanUserAcc(self, i, "v"))
            if (not rv):
                self.count += 1
                pass
            rv = mc.channel_get_access(i, "nonvolatile",
                                       MCChanUserAcc(self, i, "n"))
            if (not rv):
                self.count += 1
                pass
            pass
        return

    def add_info(self, ch, oinfo):
        info = oinfo.info
        item = self.Append(ch, "Info", [])
        v = [ 0 ]
        rv = info.get_medium(v)
        if (not rv):
            self.add_data(item, "Medium",
                          [OpenIPMI.channel_medium_string(v[0])])
            pass
        rv = info.get_protocol_type(v)
        if (not rv):
            self.add_data(item, "Protocol Type",
                          [OpenIPMI.channel_protocol_string(v[0])])
            pass
        rv = info.get_session_support(v)
        if (not rv):
            self.add_data(item, "Session Support",
                          [OpenIPMI.channel_session_support_string(v[0])])
            pass
        v = info.get_vendor_id()
        if (v):
            self.add_data(item, "Vendor ID", [v])
            pass
        v = info.get_aux_info()
        if (v):
            self.add_data(item, "Aux Info", [v])
            pass
        return
    
    def add_access(self, ch, oinfo):
        info = oinfo.access
        item = self.Append(ch, "User Access (" + oinfo.tstr + ")", [], oinfo)
        v = [ 0 ]
        rv = info.get_alerting_enabled(v)
        if (not rv):
            mitem = self.add_data(item, "Alerting Enabled", [str(v[0] != 0)],
                                  BoolSetter(self, info.set_alerting_enabled))
            pass
        rv = info.get_per_msg_auth(v)
        if (not rv):
            mitem = self.add_data(item, "Per Msg Auth", [str(v[0] != 0)],
                                  BoolSetter(self, info.set_per_msg_auth))
            pass
        rv = info.get_user_auth(v)
        if (not rv):
            mitem = self.add_data(item, "User Auth", [str(v[0] != 0)],
                                  BoolSetter(self, info.set_user_auth))
            pass
        rv = info.get_access_mode(v)
        if (not rv):
            mitem = self.add_data(item, "Access Mode",
                                  [OpenIPMI.channel_access_mode_string(v[0])],
                                  AccessSetter(self, info.set_access_mode))
            pass
        rv = info.get_privilege_limit(v)
        if (not rv):
            mitem = self.add_data(item, "Privilege Limit",
                                  [OpenIPMI.privilege_string(v[0])],
                                  PrivSetter(self, info.set_privilege_limit))
            pass
        return
    
    def setup(self):
        for i in range(0, OpenIPMI.MAX_USED_CHANNELS):
            chi = self.info[i]
            if (len(chi) > 0):
                cdata = MCChanData(self, i)
                ch = self.Append(self.treeroot, str(i), [], cdata)
                if ("info" in chi):
                    info = chi["info"]
                    v = [ 0 ]
                    rv = info.info.get_medium(v)
                    if (not rv):
                        cdata.medium = v[0]
                        s = OpenIPMI.channel_medium_string(v[0])
                        self.SetColumn(ch, s, 1)
                        pass
                    self.add_info(ch, info)
                    pass
                if ("v" in chi):
                    self.add_access(ch, chi["v"])
                    pass
                if ("n" in chi):
                    self.add_access(ch, chi["n"])
                    pass
                pass
            pass

        self.AfterDone()
        return
        
    def done_one(self):
        self.count -= 1
        if (self.count == 0):
            self.setup()
        return

    def cancel(self):
        self.Close()
        return
    
    def save(self):
        self.mc_id.to_mc(self)
        return

    def mc_cb(self, mc):
        # FIXME - add error handling
        for i in range(0, OpenIPMI.MAX_USED_CHANNELS):
            chi = self.info[i]
            if (len(chi) > 0):
                if ("v" in chi):
                    mc.channel_set_access(chi["v"].access, i, "volatile")
                    pass
                if ("n" in chi):
                    mc.channel_set_access(chi["n"].access, i, "nonvolatile")
                    pass
                pass
            pass
        self.Close()
        return

    def do_on_close(self):
        self.mc_id = None
        self.info = None
        return

    pass
